@@ -1,3 +1,23 @@
+/* FCE Ultra - NES/Famicom Emulator
+ *
+ * Copyright notice for this file:
+ *  Copyright (C) 2020 mjbudd77
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+//
 // SymbolicDebug.cpp
 #include <stdio.h>
 #include <stdlib.h>
@@ -15,692 +35,9 @@
 #include "Qt/fceuWrapper.h"
 #include "Qt/SymbolicDebug.h"
 #include "Qt/ConsoleUtilities.h"
+#include "Qt/ConsoleWindow.h"
 
-debugSymbolTable_t  debugSymbolTable;
 
-//--------------------------------------------------------------
-// debugSymbolPage_t
-//--------------------------------------------------------------
-debugSymbolPage_t::debugSymbolPage_t(void)
-{
-	pageNum = -1;
-
-}
-//--------------------------------------------------------------
-debugSymbolPage_t::~debugSymbolPage_t(void)
-{
-	std::map <int, debugSymbol_t*>::iterator it;
-
-	for (it=symMap.begin(); it!=symMap.end(); it++)
-	{
-		delete it->second;
-	}
-	symMap.clear();
-}
-//--------------------------------------------------------------
-int debugSymbolPage_t::addSymbol( debugSymbol_t*sym )
-{
-	std::map <int, debugSymbol_t*>::iterator it;
-
-	it = symMap.find( sym->ofs );
-
-	if ( it != symMap.end() )
-	{
-		return -1;
-	}
-	symMap[ sym->ofs ] = sym;
-
-	return 0;
-}
-//--------------------------------------------------------------
-debugSymbol_t *debugSymbolPage_t::getSymbolAtOffset( int ofs )
-{
-	debugSymbol_t*sym = NULL;
-	std::map <int, debugSymbol_t*>::iterator it;
-
-	it = symMap.find( ofs );
-
-	if ( it != symMap.end() )
-	{
-		sym = it->second;
-	}
-	return sym;
-}
-//--------------------------------------------------------------
-int debugSymbolPage_t::deleteSymbolAtOffset( int ofs )
-{
-	debugSymbol_t*sym = NULL;
-	std::map <int, debugSymbol_t*>::iterator it;
-
-	it = symMap.find( ofs );
-
-	if ( it != symMap.end() )
-	{
-		sym = it->second;
-		symMap.erase(it);
-	}
-	else
-	{
-		return -1;
-	}
-	if ( sym != NULL )
-	{
-		delete sym;
-	}
-	return 0;
-}
-//--------------------------------------------------------------
-int debugSymbolPage_t::save(void)
-{
-	FILE *fp;
-	debugSymbol_t *sym;
-	std::map <int, debugSymbol_t*>::iterator it;
-	const char *romFile;
-	char stmp[512];
-	int i,j;
-
-	romFile = getRomFile();
-
-	if ( romFile == NULL )
-	{
-		return -1;
-	}
-	i=0;
-	while ( romFile[i] != 0 )
-	{
-
-		if ( romFile[i] == '|' )
-		{
-			stmp[i] = '.';
-		}
-		else
-		{
-			stmp[i] = romFile[i];
-		}
-		i++;
-	}
-	stmp[i] = 0;
-
-	if ( pageNum < 0 )
-	{
-		strcat( stmp, ".ram.nl" );
-	}
-	else
-	{
-		char suffix[32];
-
-		sprintf( suffix, ".%X.nl", pageNum );
-
-		strcat( stmp, suffix );
-	}
-
-	fp = fopen( stmp, "w" );
-
-	if ( fp == NULL )
-	{
-		printf("Error: Could not open file '%s' for writing\n", stmp );
-		return -1;
-	}
-
-	for (it=symMap.begin(); it!=symMap.end(); it++)
-	{
-		const char *c;
-
-		sym = it->second;
-
-		i=0; j=0; c = sym->comment.c_str();
-		
-		while ( c[i] != 0 )
-		{
-			if ( c[i] == '\n' )
-			{
-				i++; break;
-			}
-			else
-			{
-				stmp[j] = c[i]; j++; i++;
-			}
-		}
-		stmp[j] = 0;
-
-		fprintf( fp, "$%04X#%s#%s\n", sym->ofs, sym->name.c_str(), stmp );
-
-		j=0;
-		while ( c[i] != 0 )
-		{
-			if ( c[i] == '\n' )
-			{
-				i++; stmp[j] = 0;
-
-				if ( j > 0 )
-				{
-					fprintf( fp, "\\%s\n", stmp );
-				}
-				j=0;
-			}
-			else
-			{
-				stmp[j] = c[i]; j++; i++;
-			}
-		}
-	}
-
-	fclose(fp);
-
-	return 0;
-}
-//--------------------------------------------------------------
-void debugSymbolPage_t::print(void)
-{
-	FILE *fp;
-	debugSymbol_t *sym;
-	std::map <int, debugSymbol_t*>::iterator it;
-
-	fp = stdout;
-
-	fprintf( fp, "Page: %X \n", pageNum );
-
-	for (it=symMap.begin(); it!=symMap.end(); it++)
-	{
-		sym = it->second;
-
-		fprintf( fp, "   Sym: $%04X '%s' \n", sym->ofs, sym->name.c_str() );
-	}
-}
-//--------------------------------------------------------------
-// debugSymbolTable_t
-//--------------------------------------------------------------
-debugSymbolTable_t::debugSymbolTable_t(void)
-{
-
-}
-//--------------------------------------------------------------
-debugSymbolTable_t::~debugSymbolTable_t(void)
-{
-	this->clear();
-}
-//--------------------------------------------------------------
-void debugSymbolTable_t::clear(void)
-{
-	std::map <int, debugSymbolPage_t*>::iterator it;
-
-	for (it=pageMap.begin(); it!=pageMap.end(); it++)
-	{
-		delete it->second;
-	}
-	pageMap.clear();
-}
-//--------------------------------------------------------------
-int generateNLFilenameForAddress(int address, char *NLfilename)
-{
-	int bank;
-
-	if (address < 0x8000)
-	{
-		bank = -1;
-	} 
-	else
-	{
-		bank = getBank(address);
-		#ifdef DW3_NL_0F_1F_HACK
-		if(bank == 0x0F)
-			bank = 0x1F;
-		#endif
-	}
-	return generateNLFilenameForBank( bank, NLfilename );
-}
-//--------------------------------------------------------------
-int generateNLFilenameForBank(int bank, char *NLfilename)
-{
-	int i;
-	const char *romFile;
-
-	romFile = getRomFile();
-
-	if ( romFile == NULL )
-	{
-		return -1;
-	}
-	i=0;
-	while ( romFile[i] != 0 )
-	{
-
-		if ( romFile[i] == '|' )
-		{
-			NLfilename[i] = '.';
-		}
-		else
-		{
-			NLfilename[i] = romFile[i];
-		}
-		i++;
-	}
-	NLfilename[i] = 0;
-
-	if (bank < 0)
-	{
-		// The NL file for the RAM addresses has the name nesrom.nes.ram.nl
-		strcat(NLfilename, ".ram.nl");
-	} 
-	else
-	{
-		char stmp[64];
-		#ifdef DW3_NL_0F_1F_HACK
-		if(bank == 0x0F)
-			bank = 0x1F;
-		#endif
-		sprintf( stmp, ".%X.nl", bank);
-		strcat(NLfilename, stmp );
-	}
-	return 0;
-}
-//--------------------------------------------------------------
-int debugSymbolTable_t::loadFileNL( int bank )
-{
-	FILE *fp;
-	int i, j, ofs, lineNum = 0, literal = 0, array = 0;
-	char fileName[512], line[512];
-	char stmp[512];
-	debugSymbolPage_t *page = NULL;
-	debugSymbol_t *sym = NULL;
-
-	//printf("Looking to Load Debug Bank: $%X \n", bank );
-
-	if ( generateNLFilenameForBank( bank, fileName ) )
-	{
-		return -1;
-	}
-	//printf("Loading NL File: %s\n", fileName );
-
-	fp = ::fopen( fileName, "r" );
-
-	if ( fp == NULL )
-	{
-		return -1;
-	}
-	page = new debugSymbolPage_t;
-
-	page->pageNum = bank;
-
-	pageMap[ page->pageNum ] = page;
-
-	while ( fgets( line, sizeof(line), fp ) != 0 )
-	{
-		i=0; lineNum++;
-		//printf("%4i:%s", lineNum, line );
-
-		if ( line[i] == '\\' )
-		{
-			// Line is a comment continuation line.
-			i++;
-
-			j=0;
-			stmp[j] = '\n'; j++;
-
-			while ( line[i] != 0 )
-			{
-				stmp[j] = line[i]; j++; i++;
-			}
-			stmp[j] = 0;
-
-			j--;
-			while ( j >= 0 )
-			{
-				if ( isspace( stmp[j] ) )
-				{
-					stmp[j] = 0; 
-				}
-				else
-				{
-					break;
-				}
-				j--;
-			}
-			if ( sym != NULL )
-			{
-				sym->comment.append( stmp );
-			}
-		}
-		else if ( line[i] == '$' )
-		{
-			// Line is a new debug offset
-			array = 0;
-
-			j=0; i++;
-			if ( !isxdigit( line[i] ) )
-			{
-				printf("Error: Invalid Offset on Line %i of File %s\n", lineNum, fileName );
-			}
-			while ( isxdigit( line[i] ) )
-			{
-				stmp[j] = line[i]; i++; j++;
-			}
-			stmp[j] = 0;
-
-			ofs = strtol( stmp, NULL, 16 );
-
-			if ( line[i] == '/' )
-			{
-				j=0; i++;
-				while ( isxdigit( line[i] ) )
-				{
-					stmp[j] = line[i]; i++; j++;
-				}
-				stmp[j] = 0;
-
-				array = strtol( stmp, NULL, 16 );
-			}
-
-			if ( line[i] != '#' )
-			{
-				printf("Error: Missing field delimiter following offset $%X on Line %i of File %s\n", ofs, lineNum, fileName );
-				continue;
-			}
-			i++;
-
-			while ( isspace(line[i]) ) i++;
-
-			j = 0;
-			while ( line[i] != 0 )
-			{
-				if ( line[i] == '\\' )
-				{
-					if ( literal )
-					{
-						switch ( line[i] )
-						{
-							case 'r':
-								stmp[j] = '\r';
-							break;
-							case 'n':
-								stmp[j] = '\n';
-							break;
-							case 't':
-								stmp[j] = '\t';
-							break;
-							default:
-								stmp[j] = line[i];
-							break;
-						}
-						j++; i++;
-						literal = 0;
-					}
-					else
-					{
-						i++;
-						literal = !literal;
-					}
-				}
-				else if ( line[i] == '#' )
-				{
-					break;
-				}
-				else
-				{
-					stmp[j] = line[i]; j++; i++;
-				}
-			}
-			stmp[j] = 0;
-
-			j--;
-			while ( j >= 0 )
-			{
-				if ( isspace( stmp[j] ) )
-				{
-					stmp[j] = 0; 
-				}
-				else
-				{
-					break;
-				}
-				j--;
-			}
-
-			if ( line[i] != '#' )
-			{
-				printf("Error: Missing field delimiter following name '%s' on Line %i of File %s\n", stmp, lineNum, fileName );
-				continue;
-			}
-			i++;
-
-			sym = new debugSymbol_t();
-
-			if ( sym == NULL )
-			{
-				printf("Error: Failed to allocate memory for offset $%04X Name '%s' on Line %i of File %s\n", ofs, stmp, lineNum, fileName );
-				continue;
-			}
-			sym->ofs = ofs;
-			sym->name.assign( stmp );
-			
-			while ( isspace( line[i] ) ) i++;
-
-			j=0;
-			while ( line[i] != 0 )
-			{
-				stmp[j] = line[i]; j++; i++;
-			}
-			stmp[j] = 0;
-
-			j--;
-			while ( j >= 0 )
-			{
-				if ( isspace( stmp[j] ) )
-				{
-					stmp[j] = 0; 
-				}
-				else
-				{
-					break;
-				}
-				j--;
-			}
-
-			sym->comment.assign( stmp );
-
-			if ( array > 0 )
-			{
-				debugSymbol_t *arraySym = NULL;
-
-				for (j=0; j<array; j++)
-				{
-					arraySym = new debugSymbol_t();
-
-					if ( arraySym )
-					{
-						arraySym->ofs = sym->ofs + j;
-
-						sprintf( stmp, "[%i]", j );
-						arraySym->name.assign( sym->name );
-						arraySym->name.append( stmp );
-						arraySym->comment.assign( sym->comment );
-
-						if ( page->addSymbol( arraySym ) )
-						{
-							printf("Error: Failed to add symbol for offset $%04X Name '%s' on Line %i of File %s\n", ofs, arraySym->name.c_str(), lineNum, fileName );
-							delete arraySym; arraySym = NULL; // Failed to add symbol
-						}
-					}
-				}
-				delete sym; sym = NULL; // Delete temporary symbol
-			}
-			else
-			{
-				if ( page->addSymbol( sym ) )
-				{
-					printf("Error: Failed to add symbol for offset $%04X Name '%s' on Line %i of File %s\n", ofs, sym->name.c_str(), lineNum, fileName );
-					delete sym; sym = NULL; // Failed to add symbol
-				}
-			}
-		}
-	}
-
-	::fclose(fp);
-
-	return 0;
-}
-//--------------------------------------------------------------
-int debugSymbolTable_t::loadRegisterMap(void)
-{
-	debugSymbolPage_t *page;
-
-	page = new debugSymbolPage_t();
-
-	page->pageNum = -2;
-
-	page->addSymbol( new debugSymbol_t( 0x2000, "PPU_CTRL" ) );
-	page->addSymbol( new debugSymbol_t( 0x2001, "PPU_MASK" ) );
-	page->addSymbol( new debugSymbol_t( 0x2002, "PPU_STATUS" ) );
-	page->addSymbol( new debugSymbol_t( 0x2003, "PPU_OAM_ADDR" ) );
-	page->addSymbol( new debugSymbol_t( 0x2004, "PPU_OAM_DATA" ) );
-	page->addSymbol( new debugSymbol_t( 0x2005, "PPU_SCROLL" ) );
-	page->addSymbol( new debugSymbol_t( 0x2006, "PPU_ADDRESS" ) );
-	page->addSymbol( new debugSymbol_t( 0x2007, "PPU_DATA" ) );
-	page->addSymbol( new debugSymbol_t( 0x4000, "SQ1_VOL" ) );
-	page->addSymbol( new debugSymbol_t( 0x4001, "SQ1_SWEEP" ) );
-	page->addSymbol( new debugSymbol_t( 0x4002, "SQ1_LO" ) );
-	page->addSymbol( new debugSymbol_t( 0x4003, "SQ1_HI" ) );
-	page->addSymbol( new debugSymbol_t( 0x4004, "SQ2_VOL" ) );
-	page->addSymbol( new debugSymbol_t( 0x4005, "SQ2_SWEEP" ) );
-	page->addSymbol( new debugSymbol_t( 0x4006, "SQ2_LO" ) );
-	page->addSymbol( new debugSymbol_t( 0x4007, "SQ2_HI" ) );
-	page->addSymbol( new debugSymbol_t( 0x4008, "TRI_LINEAR" ) );
-//	page->addSymbol( new debugSymbol_t( 0x4009, "UNUSED" ) );
-	page->addSymbol( new debugSymbol_t( 0x400A, "TRI_LO" ) );
-	page->addSymbol( new debugSymbol_t( 0x400B, "TRI_HI" ) );
-	page->addSymbol( new debugSymbol_t( 0x400C, "NOISE_VOL" ) );
-//	page->addSymbol( new debugSymbol_t( 0x400D, "UNUSED" ) );
-	page->addSymbol( new debugSymbol_t( 0x400E, "NOISE_LO" ) );
-	page->addSymbol( new debugSymbol_t( 0x400F, "NOISE_HI" ) );
-	page->addSymbol( new debugSymbol_t( 0x4010, "DMC_FREQ" ) );
-	page->addSymbol( new debugSymbol_t( 0x4011, "DMC_RAW" ) );
-	page->addSymbol( new debugSymbol_t( 0x4012, "DMC_START" ) );
-	page->addSymbol( new debugSymbol_t( 0x4013, "DMC_LEN" ) );
-	page->addSymbol( new debugSymbol_t( 0x4014, "OAM_DMA" ) );
-	page->addSymbol( new debugSymbol_t( 0x4015, "APU_STATUS" ) );
-	page->addSymbol( new debugSymbol_t( 0x4016, "JOY1" ) );
-	page->addSymbol( new debugSymbol_t( 0x4017, "JOY2_FRAME" ) );
-
-	pageMap[ page->pageNum ] = page;
-
-	return 0;
-}
-//--------------------------------------------------------------
-int debugSymbolTable_t::loadGameSymbols(void)
-{
-	int nPages, pageSize, romSize = 0x10000;
-
-	this->save();
-	this->clear();
-
-	if ( GameInfo != NULL )
-	{
-		romSize = 16 + CHRsize[0] + PRGsize[0];
-	}
-
-	loadFileNL( -1 );
-
-	loadRegisterMap();
-
-	pageSize = (1<<debuggerPageSize);
-
-	//nPages = 1<<(15-debuggerPageSize);
-	nPages = romSize / pageSize;
-
-	//printf("RomSize: %i    NumPages: %i \n", romSize, nPages );
-
-	for(int i=0;i<nPages;i++)
-	{
-		//printf("Loading Page Offset: $%06X\n", pageSize*i );
-
-		loadFileNL( i );
-	}
-
-	//print();
-
-	return 0;
-}
-//--------------------------------------------------------------
-int debugSymbolTable_t::addSymbolAtBankOffset( int bank, int ofs, debugSymbol_t *sym )
-{
-	debugSymbolPage_t *page;
-	std::map <int, debugSymbolPage_t*>::iterator it;
-
-	it = pageMap.find( bank );
-
-	if ( it == pageMap.end() )
-	{
-		page = new debugSymbolPage_t();
-		page->pageNum = bank;
-		pageMap[ bank ] = page;
-	}
-	else
-	{
-		page = it->second;
-	}
-	page->addSymbol( sym );
-
-	return 0;
-}
-//--------------------------------------------------------------
-int debugSymbolTable_t::deleteSymbolAtBankOffset( int bank, int ofs )
-{
-	debugSymbolPage_t *page;
-	std::map <int, debugSymbolPage_t*>::iterator it;
-
-	it = pageMap.find( bank );
-
-	if ( it == pageMap.end() )
-	{
-		return -1;
-	}
-	else
-	{
-		page = it->second;
-	}
-
-	return page->deleteSymbolAtOffset( ofs );
-}
-//--------------------------------------------------------------
-debugSymbol_t *debugSymbolTable_t::getSymbolAtBankOffset( int bank, int ofs )
-{
-	debugSymbol_t*sym = NULL;
-	std::map <int, debugSymbolPage_t*>::iterator it;
-
-	it = pageMap.find( bank );
-
-	if ( it != pageMap.end() )
-	{
-		sym = (it->second)->getSymbolAtOffset( ofs );
-	}
-	return sym;
-}
-//--------------------------------------------------------------
-void debugSymbolTable_t::save(void)
-{
-	debugSymbolPage_t *page;
-	std::map <int, debugSymbolPage_t*>::iterator it;
-
-	for (it=pageMap.begin(); it!=pageMap.end(); it++)
-	{
-		page = it->second;
-
-		page->save();
-	}
-}
-//--------------------------------------------------------------
-void debugSymbolTable_t::print(void)
-{
-	debugSymbolPage_t *page;
-	std::map <int, debugSymbolPage_t*>::iterator it;
-
-	for (it=pageMap.begin(); it!=pageMap.end(); it++)
-	{
-		page = it->second;
-
-		page->print();
-	}
-}
 //--------------------------------------------------------------
 debugSymbol_t *replaceSymbols( int flags, int addr, char *str )
 {
@@ -726,30 +63,30 @@ debugSymbol_t *replaceSymbols( int flags, int addr, char *str )
 	{
 		if ( flags & ASM_DEBUG_REPLACE )
 		{
-			strcpy( str, sym->name.c_str() );
+			strcpy( str, sym->name().c_str() );
 		}
 		else
 		{
 			if ( flags & ASM_DEBUG_ADDR_02X )
 			{
-				sprintf( str, "%02X ", addr );
+				sprintf( str, "$%02X ", addr );
 			}
 			else
 			{
-				sprintf( str, "%04X ", addr );
+				sprintf( str, "$%04X ", addr );
 			}
-			strcat( str, sym->name.c_str() );
+			strcat( str, sym->name().c_str() );
 		}
 	}
 	else
 	{
 		if ( flags & ASM_DEBUG_ADDR_02X )
 		{
-			sprintf( str, "%02X", addr );
+			sprintf( str, "$%02X", addr );
 		}
 		else
 		{
-			sprintf( str, "%04X", addr );
+			sprintf( str, "$%04X", addr );
 		}
 	}
 
@@ -763,6 +100,10 @@ int DisassembleWithDebug(int addr, uint8_t *opcode, int flags, char *str, debugS
 	static char chr[8]={0};
 	uint16_t tmp,tmp2;
 	char stmp[128], stmp2[128];
+	bool symDebugEnable, showTrace;
+
+	symDebugEnable = (flags & ASM_DEBUG_SYMS  ) ? true : false;
+	showTrace      = (flags & ASM_DEBUG_TRACES) ? true : false;
 
 	//these may be replaced later with passed-in values to make a lighter-weight disassembly mode that may not query the referenced values
 	#define RX (X.X)
@@ -840,14 +181,18 @@ int DisassembleWithDebug(int addr, uint8_t *opcode, int flags, char *str, debugS
 		_indirectx:
 			indirectX(tmp);
 
-			if ( flags )
+			if ( symDebugEnable )
 			{
 				sym = replaceSymbols( flags, tmp, stmp );
-				sprintf(str,"%s ($%02X,X) @ $%s = #$%02X", chr,opcode[1],stmp,GetMem(tmp));
+				showTrace
+					? sprintf(str,"%s ($%02X,X) @ %s = #$%02X", chr,opcode[1],stmp,GetMem(tmp))
+					: sprintf(str,"%s ($%02X,X)", chr,opcode[1]);
 			}
 			else
 			{
-				sprintf(str,"%s ($%02X,X) @ $%04X = #$%02X", chr,opcode[1],tmp,GetMem(tmp));
+				showTrace
+					? sprintf(str,"%s ($%02X,X) @ $%04X = #$%02X", chr,opcode[1],tmp,GetMem(tmp))
+					: sprintf(str,"%s ($%02X,X)", chr,opcode[1]);
 			}
 			break;
 
@@ -876,14 +221,18 @@ int DisassembleWithDebug(int addr, uint8_t *opcode, int flags, char *str, debugS
 		_zeropage:
 		// ################################## Start of SP CODE ###########################
 		// Change width to %04X // don't!
-			if ( flags )
+			if ( symDebugEnable )
 			{
 				sym = replaceSymbols( flags | ASM_DEBUG_ADDR_02X, opcode[1], stmp );
-				sprintf(str,"%s $%s = #$%02X", chr,stmp,GetMem(opcode[1]));
+				showTrace
+					? sprintf(str,"%s %s = #$%02X", chr,stmp,GetMem(opcode[1]))
+					: sprintf(str,"%s %s", chr,stmp);
 			}
 			else
 			{
-				sprintf(str,"%s $%02X = #$%02X", chr,opcode[1],GetMem(opcode[1]));
+				showTrace
+					? sprintf(str,"%s $%02X = #$%02X", chr,opcode[1],GetMem(opcode[1]))
+					: sprintf(str,"%s $%02X", chr,opcode[1]);
 			}
 		// ################################## End of SP CODE ###########################
 			break;
@@ -930,14 +279,18 @@ int DisassembleWithDebug(int addr, uint8_t *opcode, int flags, char *str, debugS
 		_absolute:
 			absolute(tmp);
 
-			if ( flags )
+			if ( symDebugEnable )
 			{
 				sym = replaceSymbols( flags, tmp, stmp );
-				sprintf(str,"%s $%s = #$%02X", chr,stmp,GetMem(tmp));
+				showTrace
+					? sprintf(str,"%s %s = #$%02X", chr,stmp,GetMem(tmp))
+					: sprintf(str,"%s %s", chr,stmp);
 			}
 			else
 			{
-				sprintf(str,"%s $%04X = #$%02X", chr,tmp,GetMem(tmp));
+				showTrace
+					? sprintf(str,"%s $%04X = #$%02X", chr,tmp,GetMem(tmp))
+					: sprintf(str,"%s $%04X", chr,tmp);
 			}
 			break;
 
@@ -953,10 +306,10 @@ int DisassembleWithDebug(int addr, uint8_t *opcode, int flags, char *str, debugS
 		_branch:
 			relative(tmp);
 
-			if ( flags )
+			if ( symDebugEnable )
 			{
 				sym = replaceSymbols( flags, tmp, stmp );
-				sprintf(str,"%s $%s", chr,stmp);
+				sprintf(str,"%s %s", chr,stmp);
 			}
 			else
 			{
@@ -976,14 +329,18 @@ int DisassembleWithDebug(int addr, uint8_t *opcode, int flags, char *str, debugS
 		_indirecty:
 			indirectY(tmp);
 
-			if ( flags )
+			if ( symDebugEnable )
 			{
 				sym = replaceSymbols( flags, tmp, stmp );
-				sprintf(str,"%s ($%02X),Y @ $%s = #$%02X", chr,opcode[1],stmp,GetMem(tmp));
+				showTrace
+					? sprintf(str,"%s ($%02X),Y @ %s = #$%02X", chr,opcode[1],stmp,GetMem(tmp))
+					: sprintf(str,"%s ($%02X),Y", chr,opcode[1]);
 			}
 			else
 			{
-				sprintf(str,"%s ($%02X),Y @ $%04X = #$%02X", chr,opcode[1],tmp,GetMem(tmp));
+				showTrace
+					? sprintf(str,"%s ($%02X),Y @ $%04X = #$%02X", chr,opcode[1],tmp,GetMem(tmp))
+					: sprintf(str,"%s ($%02X),Y", chr,opcode[1]);
 			}
 			break;
 
@@ -1008,14 +365,18 @@ int DisassembleWithDebug(int addr, uint8_t *opcode, int flags, char *str, debugS
 			zpIndex(tmp,RX);
 		// ################################## Start of SP CODE ###########################
 		// Change width to %04X // don't!
-			if ( flags )
+			if ( symDebugEnable )
 			{
 				sym = replaceSymbols( flags, tmp, stmp );
-				sprintf(str,"%s $%02X,X @ $%s = #$%02X", chr,opcode[1],stmp,GetMem(tmp));
+				showTrace
+					? sprintf(str,"%s $%02X,X @ %s = #$%02X", chr,opcode[1],stmp,GetMem(tmp))
+					: sprintf(str,"%s $%02X,X", chr,opcode[1]);
 			}
 			else
 			{
-				sprintf(str,"%s $%02X,X @ $%04X = #$%02X", chr,opcode[1],tmp,GetMem(tmp));
+				showTrace
+					? sprintf(str,"%s $%02X,X @ $%04X = #$%02X", chr,opcode[1],tmp,GetMem(tmp))
+					: sprintf(str,"%s $%02X,X", chr,opcode[1]);
 			}
 		// ################################## End of SP CODE ###########################
 			break;
@@ -1033,15 +394,19 @@ int DisassembleWithDebug(int addr, uint8_t *opcode, int flags, char *str, debugS
 		_absolutey:
 			absolute(tmp);
 			tmp2=(tmp+RY);
-			if ( flags )
+			if ( symDebugEnable )
 			{
 				sym  = replaceSymbols( flags, tmp , stmp  );
 				sym2 = replaceSymbols( flags, tmp2, stmp2 );
-				sprintf(str,"%s $%s,Y @ $%s = #$%02X", chr,stmp,stmp2,GetMem(tmp2));
+				showTrace
+					? sprintf(str,"%s %s,Y @ %s = #$%02X", chr,stmp,stmp2,GetMem(tmp2))
+					: sprintf(str,"%s %s,Y", chr,stmp);
 			}
 			else
 			{
-				sprintf(str,"%s $%04X,Y @ $%04X = #$%02X", chr,tmp,tmp2,GetMem(tmp2));
+				showTrace
+					? sprintf(str,"%s $%04X,Y @ $%04X = #$%02X", chr,tmp,tmp2,GetMem(tmp2))
+					: sprintf(str,"%s $%04X,Y", chr,tmp);
 			}
 			break;
 
@@ -1064,15 +429,19 @@ int DisassembleWithDebug(int addr, uint8_t *opcode, int flags, char *str, debugS
 		_absolutex:
 			absolute(tmp);
 			tmp2=(tmp+RX);
-			if ( flags )
+			if ( symDebugEnable )
 			{
 				sym  = replaceSymbols( flags, tmp , stmp  );
 				sym2 = replaceSymbols( flags, tmp2, stmp2 );
-				sprintf(str,"%s $%s,X @ $%s = #$%02X", chr,stmp,stmp2,GetMem(tmp2));
+				showTrace
+					? sprintf(str,"%s %s,X @ %s = #$%02X", chr,stmp,stmp2,GetMem(tmp2))
+					: sprintf(str,"%s %s,X", chr,stmp);
 			}
 			else
 			{
-				sprintf(str,"%s $%04X,X @ $%04X = #$%02X", chr,tmp,tmp2,GetMem(tmp2));
+				showTrace
+					? sprintf(str,"%s $%04X,X @ $%04X = #$%02X", chr,tmp,tmp2,GetMem(tmp2))
+					: sprintf(str,"%s $%04X,X", chr,tmp);
 			}
 			break;
 
@@ -1083,10 +452,10 @@ int DisassembleWithDebug(int addr, uint8_t *opcode, int flags, char *str, debugS
 		_jump:
 			absolute(tmp);
 
-			if ( flags )
+			if ( symDebugEnable )
 			{
 				sym = replaceSymbols( flags, tmp, stmp );
-				sprintf(str,"%s $%s", chr,stmp);
+				sprintf(str,"%s %s", chr,stmp);
 			}
 			else
 			{
@@ -1101,14 +470,18 @@ int DisassembleWithDebug(int addr, uint8_t *opcode, int flags, char *str, debugS
 			zpIndex(tmp,RY);
 		// ################################## Start of SP CODE ###########################
 		// Change width to %04X // don't!
-			if ( flags )
+			if ( symDebugEnable )
 			{
 				sym = replaceSymbols( flags, tmp, stmp );
-				sprintf(str,"%s $%02X,Y @ $%s = #$%02X", chr,opcode[1],stmp,GetMem(tmp));
+				showTrace
+					? sprintf(str,"%s $%02X,Y @ %s = #$%02X", chr,opcode[1],stmp,GetMem(tmp))
+					: sprintf(str,"%s $%02X,Y", chr,opcode[1]);
 			}
 			else
 			{
-				sprintf(str,"%s $%02X,Y @ $%04X = #$%02X", chr,opcode[1],tmp,GetMem(tmp));
+				showTrace
+					? sprintf(str,"%s $%02X,Y @ $%04X = #$%02X", chr,opcode[1],tmp,GetMem(tmp))
+					: sprintf(str,"%s $%02X,Y", chr,opcode[1]);
 			}
 		// ################################## End of SP CODE ###########################
 			break;
@@ -1269,15 +642,15 @@ SymbolEditWindow::SymbolEditWindow(QWidget *parent)
 	hbox->addWidget(     okButton );
 
 	connect(     okButton, SIGNAL(clicked(void)), this, SLOT(accept(void)) );
-   connect( cancelButton, SIGNAL(clicked(void)), this, SLOT(reject(void)) );
+	connect( cancelButton, SIGNAL(clicked(void)), this, SLOT(reject(void)) );
 
 	deleteBox->setEnabled( false );
 	okButton->setDefault(true);
 
 	if ( sym != NULL )
 	{
-		nameEntry->setText( tr(sym->name.c_str()) );
-		commentEntry->setPlainText( tr(sym->comment.c_str()) );
+		nameEntry->setText( tr(sym->name().c_str()) );
+		commentEntry->setPlainText( tr(sym->comment().c_str()) );
 	}
 
 	setLayout( mainLayout );
@@ -1295,16 +668,16 @@ SymbolEditWindow::~SymbolEditWindow(void)
 //--------------------------------------------------------------
 void SymbolEditWindow::closeEvent(QCloseEvent *event)
 {
-   printf("Debugger Close Window Event\n");
-   done(0);
+	//printf("Symbolic Debug Close Window Event\n");
+	done(0);
 	deleteLater();
-   event->accept();
+	event->accept();
 }
 //--------------------------------------------------------------
 void SymbolEditWindow::closeWindow(void)
 {
-   //printf("Close Window\n");
-   done(0);
+	//printf("Close Window\n");
+	done(0);
 	deleteLater();
 }
 //--------------------------------------------------------------
@@ -1348,7 +721,9 @@ void SymbolEditWindow::arrayCommentHeadOnlyChanged( int state )
 //--------------------------------------------------------------
 void SymbolEditWindow::setAddr( int addrIn )
 {
-	char stmp[512];
+	char stmp[64];
+	std::string filename;
+	size_t size;
 
 	addr = addrIn;
 
@@ -1368,10 +743,16 @@ void SymbolEditWindow::setAddr( int addrIn )
 		}
 	}
 
-	generateNLFilenameForAddress( addr, stmp );
+	generateNLFilenameForAddress( addr, filename );
 
-	filepath->setText( tr(stmp) );
-	filepath->setMinimumWidth( charWidth * (filepath->text().size() + 4) );
+	filepath->setText( tr(filename.c_str()) );
+
+	size = filepath->text().size();
+
+	// Limit max size so that widget size doesn't explode on a large file path.
+	if (size > 32) size = 32;
+
+	filepath->setMinimumWidth( charWidth * (size + 4) );
 }
 //--------------------------------------------------------------
 void SymbolEditWindow::setBank( int bankIn )
@@ -1385,8 +766,8 @@ void SymbolEditWindow::setSym( debugSymbol_t *symIn )
 
 	if ( sym != NULL )
 	{
-		nameEntry->setText( tr(sym->name.c_str()) );
-		commentEntry->setPlainText( tr(sym->comment.c_str()) );
+		nameEntry->setText( tr(sym->name().c_str()) );
+		commentEntry->setPlainText( tr(sym->comment().c_str()) );
 		deleteBox->setEnabled( true );
 
 		determineArrayStart();
@@ -1410,7 +791,7 @@ int SymbolEditWindow::exec(void)
 
 	if ( ret == QDialog::Accepted )
 	{
-		fceuWrapperLock();
+		FCEU_WRAPPER_LOCK();
 		if ( isArrayBox->isChecked() )
 		{
 			size = atoi( arraySize->text().toStdString().c_str() );
@@ -1435,24 +816,28 @@ int SymbolEditWindow::exec(void)
 
 				if ( deleteBox->isChecked() )
 				{
-					if ( sym != NULL )
+					if ( sym != nullptr )
 					{
 						debugSymbolTable.deleteSymbolAtBankOffset( b, a );
 					}
 				}
 				else
 				{
-					if ( sym == NULL )
+					if ( sym == nullptr )
 					{
-						sym = new debugSymbol_t();
-
-						sym->ofs = a;
-
-						debugSymbolTable.addSymbolAtBankOffset( b, a, sym );
+						sym = new debugSymbol_t(a);
 
+						if ( debugSymbolTable.addSymbolAtBankOffset( b, a, sym ) )
+						{
+							if (consoleWindow)
+							{
+								consoleWindow->QueueErrorMsgWindow( debugSymbolTable.errorMessage() );
+							}
+							delete sym;
+						}
 						isNew = true;
 					}
-					sym->ofs = a;
+					sym->setOffset(a);
 
 					if ( (i == 0) || isNew || arrayNameOverWrite->isChecked() )
 					{
@@ -1462,7 +847,7 @@ int SymbolEditWindow::exec(void)
 					{
 						if ( isNew || arrayCommentOverWrite->isChecked() || (i == 0) )
 						{
-							sym->comment = commentEntry->toPlainText().toStdString();
+							sym->commentAssign( commentEntry->toPlainText().toStdString() );
 						}
 					}
 					sym->trimTrailingSpaces();
@@ -1480,21 +865,33 @@ int SymbolEditWindow::exec(void)
 			}
 			else if ( sym == NULL )
 			{
-				sym = new debugSymbol_t();
-				sym->ofs     = addr;
-				sym->name    = nameEntry->text().toStdString();
-				sym->comment = commentEntry->toPlainText().toStdString();
+				sym = new debugSymbol_t( addr, nameEntry->text().toStdString().c_str(), 
+						commentEntry->toPlainText().toStdString().c_str());
 
-				debugSymbolTable.addSymbolAtBankOffset( bank, addr, sym );
+				if ( debugSymbolTable.addSymbolAtBankOffset( bank, addr, sym ) )
+				{
+					if (consoleWindow)
+					{
+						consoleWindow->QueueErrorMsgWindow( debugSymbolTable.errorMessage() );
+					}
+					delete sym;
+				}
 			}
 			else
 			{
-				sym->name    = nameEntry->text().toStdString();
-				sym->comment = commentEntry->toPlainText().toStdString();
+				if ( sym->updateName( nameEntry->text().toStdString().c_str() ) )
+				{
+					if (consoleWindow)
+					{
+						consoleWindow->QueueErrorMsgWindow( debugSymbolTable.errorMessage() );
+					}
+				}
+				sym->commentAssign( commentEntry->toPlainText().toStdString().c_str() );
+				sym->trimTrailingSpaces();
 			}
-			sym->trimTrailingSpaces();
 		}
-		fceuWrapperUnLock();
+		debugSymbolTable.save(); // Save table to disk immediately after an add, edit, or delete
+		FCEU_WRAPPER_UNLOCK();
 	}
 	return ret;
 }
@@ -1599,13 +996,12 @@ void SymbolEditWindow::setSymNameWithArray(int idx)
 	}
 
 	// Reform with base string and new index.
-	sym->name.assign( stmp );
-
-	sym->trimTrailingSpaces();
-
-	sprintf( stmp, "[%i]", idx );
-
-	sym->name.append( stmp );
-
+	if ( sym->updateName( stmp, idx ) )
+	{
+		if (consoleWindow)
+		{
+			consoleWindow->QueueErrorMsgWindow( debugSymbolTable.errorMessage() );
+		}
+	}
 }
 //--------------------------------------------------------------
