@@ -13,6 +13,9 @@ void WriteSound(int32 *Buffer, int Count);
 int KillSound(void);
 uint32 GetMaxSound(void);
 uint32 GetWriteSound(void);
+bool FCEUD_SoundIsMuted(void);
+void FCEUD_MuteSoundOutput(bool value);
+void FCEUD_MuteSoundWindow(bool value);
 
 void SilenceSound(int s); /* DOS and SDL */
 
@@ -20,10 +23,12 @@ int InitJoysticks(void);
 int KillJoysticks(void);
 int AddJoystick( int which );
 int RemoveJoystick( int which );
+int FindJoystickByInstanceID( int which );
 uint32 *GetJSOr(void);
 
 int InitVideo(FCEUGI *gi);
 int KillVideo(void);
+void CalcVideoDimensions(void);
 void BlitScreen(uint8 *XBuf);
 void LockConsole(void);
 void UnlockConsole(void);
