@@ -1,3 +1,22 @@
+/* FCE Ultra - NES/Famicom Emulator
+ *
+ * Copyright notice for this file:
+ *  Copyright (C) 2020 mjbudd77
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
 // fceuWrapper.cpp
 //
 #include <stdio.h>
@@ -6,6 +25,7 @@
 #include <limits.h>
 #include <unzip.h>
 
+#include <QFileInfo>
 #include <QStyleFactory>
 #include "Qt/main.h"
 #include "Qt/throttle.h"
@@ -17,16 +37,20 @@
 #include "Qt/sdl-video.h"
 #include "Qt/nes_shm.h"
 #include "Qt/unix-netplay.h"
+#include "Qt/AviRecord.h"
 #include "Qt/HexEditor.h"
+#include "Qt/CheatsConf.h"
 #include "Qt/SymbolicDebug.h"
 #include "Qt/CodeDataLogger.h"
 #include "Qt/ConsoleDebugger.h"
 #include "Qt/ConsoleWindow.h"
 #include "Qt/ConsoleUtilities.h"
+#include "Qt/TasEditor/TasEditorWindow.h"
 #include "Qt/fceux_git_info.h"
 
 #include "common/cheat.h"
 #include "../../fceu.h"
+#include "../../cheat.h"
 #include "../../movie.h"
 #include "../../version.h"
 
@@ -34,6 +58,7 @@
 #include "../../fceulua.h"
 #endif
 
+#include "common/os_utils.h"
 #include "common/configSys.h"
 #include "../../oldmovie.h"
 #include "../../types.h"
@@ -42,6 +67,11 @@
 #include "../videolog/nesvideos-piece.h"
 #endif
 
+#ifdef _MSC_VER 
+//not #if defined(_WIN32) || defined(_WIN64) because we have strncasecmp in mingw
+#define strncasecmp _strnicmp
+#define strcasecmp _stricmp
+#endif
 //*****************************************************************
 // Define Global Variables to be shared with FCEU Core
 //*****************************************************************
@@ -57,6 +87,9 @@ bool swapDuty = 0;
 bool turbo = false;
 bool pauseAfterPlayback = false;
 bool suggestReadOnlyReplay = true;
+bool showStatusIconOpt = true;
+bool drawInputAidsEnable = true;
+bool usePaletteForVideoBg = false;
 unsigned int gui_draw_area_width   = 256;
 unsigned int gui_draw_area_height  = 256;
 
@@ -69,8 +102,8 @@ static int frameskip=0;
 static int periodic_saves = 0;
 static int   mutexLocks = 0;
 static int   mutexPending = 0;
-static bool  emulatorHasMutux = 0;
-static unsigned int emulatorCycleCount = 0;
+static bool  emulatorHasMutex = 0;
+unsigned int emulatorCycleCount = 0;
 
 extern double g_fpsScale;
 
@@ -117,13 +150,37 @@ EMUFILE_FILE* FCEUD_UTF8_fstream(const char *fn, const char *m)
 	//return new std::fstream(fn,mode);
 }
 
-static const char *s_linuxCompilerString = "g++ " __VERSION__;
+#if defined(MSVC)
+ #ifdef _M_X64
+   #define _MSVC_ARCH "x64"
+ #else
+   #define _MSVC_ARCH "x86"
+ #endif
+ #ifdef _DEBUG
+  #define _MSVC_BUILD "debug"
+ #else
+  #define _MSVC_BUILD "release"
+ #endif
+ #define __COMPILER__STRING__ "msvc " _Py_STRINGIZE(_MSC_VER) " " _MSVC_ARCH " " _MSVC_BUILD
+ #define _Py_STRINGIZE(X) _Py_STRINGIZE1((X))
+ #define _Py_STRINGIZE1(X) _Py_STRINGIZE2 ## X
+ #define _Py_STRINGIZE2(X) #X
+ //re: http://72.14.203.104/search?q=cache:HG-okth5NGkJ:mail.python.org/pipermail/python-checkins/2002-November/030704.html+_msc_ver+compiler+version+string&hl=en&gl=us&ct=clnk&cd=5
+#elif defined(__GNUC__)
+ #define __COMPILER__STRING__ "gcc " __VERSION__
+#elif defined(__clang__)
+ #define __COMPILER__STRING__ "clang " __VERSION__
+#else
+ #define __COMPILER__STRING__ "unknown"
+#endif
+
+static const char *s_CompilerString = __COMPILER__STRING__;
 /**
  * Returns the compiler string.
  */
 const char *FCEUD_GetCompilerString(void)
 {
-	return s_linuxCompilerString;
+	return s_CompilerString;
 }
 
 /**
@@ -151,20 +208,29 @@ FCEUD_GetTimeFreq(void)
 static int
 DriverInitialize(FCEUGI *gi)
 {
-	if(InitVideo(gi) < 0) return 0;
+	if (InitVideo(gi) < 0)
+	{
+		return 0;
+	}
 	inited|=4;
 
-	if(InitSound())
+	if (InitSound())
+	{
 		inited|=1;
+	}
 
-	if(InitJoysticks())
+	if (InitJoysticks())
+	{
 		inited|=2;
+	}
 
 	int fourscore=0;
 	g_config->getOption("SDL.FourScore", &fourscore);
 	eoptions &= ~EO_FOURSCORE;
-	if(fourscore)
+	if (fourscore)
+	{
 		eoptions |= EO_FOURSCORE;
+	}
 
 	InitInputInterface();
 	return 1;
@@ -188,6 +254,17 @@ DriverKill()
 	inited=0;
 }
 
+int LoadGameFromLua( const char *path )
+{
+	//printf("Load From Lua: '%s'\n", path);
+	fceuWrapperUnLock();
+
+	consoleWindow->emulatorThread->signalRomLoad(path);
+
+	fceuWrapperLock();
+	return 0;
+}
+
 /**
  * Reloads last game
  */
@@ -206,23 +283,36 @@ int reloadLastGame(void)
  */
 int LoadGame(const char *path, bool silent)
 {
-	char fullpath[4096];
+	std::string fullpath;
 	int gg_enabled, autoLoadDebug, autoOpenDebugger, autoInputPreset;
 
 	if (isloaded){
 		CloseGame();
 	}
 
-#if defined(__linux) || defined(__APPLE__)
+	QFileInfo fi( path );
 
 	// Resolve absolute path to file
-	if ( realpath( path, fullpath ) == NULL )
+	if ( fi.exists() )
 	{
-		strcpy( fullpath, path );
+		//printf("FI: '%s'\n", fi.absoluteFilePath().toStdString().c_str() );
+		//printf("FI: '%s'\n", fi.canonicalFilePath().toStdString().c_str() );
+		fullpath = fi.canonicalFilePath().toStdString();
 	}
-#else
-	strcpy( fullpath, path );
-#endif
+	else
+	{
+		fullpath.assign( path );
+	}
+//#if defined(__linux__) || defined(__APPLE__) || defined(__unix__)
+//
+//	// Resolve absolute path to file
+//	if ( realpath( path, fullpath ) == NULL )
+//	{
+//		strcpy( fullpath, path );
+//	}
+//#else
+//	strcpy( fullpath, path );
+//#endif
 
 	//printf("Fullpath: %zi '%s'\n", sizeof(fullpath), fullpath );
 
@@ -233,10 +323,19 @@ int LoadGame(const char *path, bool silent)
 
 	FCEUI_SetGameGenie (gg_enabled);
 
-	if(!FCEUI_LoadGame(fullpath, 1, silent)) {
+	// Set RAM Init Method Prior to Loading New Game
+	g_config->getOption ("SDL.RamInitMethod", &RAMInitOption);
+
+	// Load the game
+	if(!FCEUI_LoadGame(fullpath.c_str(), 1, silent)) {
 		return 0;
 	}
 
+	if ( consoleWindow )
+	{
+		consoleWindow->addRecentRom( fullpath.c_str() );
+	}
+
 	hexEditorLoadBookmarks();
 
 	g_config->getOption( "SDL.AutoLoadDebugFiles", &autoLoadDebug );
@@ -255,6 +354,8 @@ int LoadGame(const char *path, bool silent)
 
 	debugSymbolTable.loadGameSymbols();
 
+	updateCheatDialog();
+
 	CDLoggerROMChanged();
 
 	int state_to_load;
@@ -279,23 +380,67 @@ int LoadGame(const char *path, bool silent)
 	}
 	
 	// set pal/ntsc
-	int id;
-	g_config->getOption("SDL.PAL", &id);
-	FCEUI_SetRegion(id);
+	int id, region, autoDetectPAL;
+	g_config->getOption("SDL.PAL", &region);
+	g_config->getOption("SDL.AutoDetectPAL", &autoDetectPAL);
+
+	if ( autoDetectPAL )
+	{
+		id = FCEUI_GetCurrentVidSystem(NULL, NULL);
+
+		if ( region == 2 )
+		{	// Dendy mode:
+			//   Run PAL Games as PAL
+			//   Run NTSC Games as Dendy
+			if ( id == 1 )
+			{
+				g_config->setOption("SDL.PAL", id);
+				FCEUI_SetRegion(id);
+			}
+			else
+			{
+				FCEUI_SetRegion(region);
+			}
+		}
+		else
+		{	// Run NTSC games as NTSC and PAL games as PAL
+			g_config->setOption("SDL.PAL", id);
+			FCEUI_SetRegion(id);
+		}
+	}
+	else
+	{
+		// If not Auto-detection of region,
+		// Strictly enforce region GUI selection
+		// Does not matter what type of game is 
+		// loaded, the current region selection is used 
+		FCEUI_SetRegion(region);
+	}
+
+	// Always re-calculate video dimensions after setting region.
+	CalcVideoDimensions();
+
+	// Force re-send of video settings to console viewer
+	if ( consoleWindow )
+	{
+		consoleWindow->videoReset();
+	}
 
 	g_config->getOption("SDL.SwapDuty", &id);
 	swapDuty = id;
 	
-	std::string filename;
-	g_config->getOption("SDL.Sound.RecordFile", &filename);
-	if(filename.size()) {
-		if(!FCEUI_BeginWaveRecord(filename.c_str())) {
-			g_config->setOption("SDL.Sound.RecordFile", "");
-		}
-	}
+	// Wave Recording done through menu or hotkeys
+	//std::string filename;
+	//g_config->getOption("SDL.Sound.RecordFile", &filename);
+	//if (filename.size()) 
+	//{
+	//	if (!FCEUI_BeginWaveRecord(filename.c_str())) {
+	//		g_config->setOption("SDL.Sound.RecordFile", "");
+	//	}
+	//}
 	isloaded = 1;
 
-	FCEUD_NetworkConnect();
+	//FCEUD_NetworkConnect();
 	return 1;
 }
 
@@ -307,23 +452,53 @@ CloseGame(void)
 {
 	std::string filename;
 
-	if(!isloaded) {
+	if ( nes_shm )
+	{	// Clear Screen on Game Close
+		nes_shm->clear_pixbuf();
+		nes_shm->blitUpdated = 1;
+	}
+
+	if (!isloaded) {
 		return(0);
 	}
+
+	// If the emulation thread is stuck hanging at a breakpoint,
+	// disable breakpoint debugging and wait for the thread to 
+	// complete its frame. So that it is idle with a minimal call
+	// stack when we close the ROM. After thread has completed the
+	// frame, it is then safe to re-enable breakpoint debugging.
+	if ( debuggerWaitingAtBreakpoint() )
+	{
+		bpDebugSetEnable(false);
+
+		if ( fceuWrapperIsLocked() )
+		{
+			fceuWrapperUnLock();
+			msleep(100);
+			fceuWrapperLock();
+		}
+		else
+		{
+			msleep(100);
+		}
+		bpDebugSetEnable(true);
+	}
+
 	hexEditorSaveBookmarks();
 	saveGameDebugBreakpoints();
 	debuggerClearAllBreakpoints();
+	debuggerClearAllBookmarks();
 
 	debugSymbolTable.save();
 	debugSymbolTable.clear();
 	CDLoggerROMClosed();
 
-   int state_to_save;
-   g_config->getOption("SDL.AutoSaveState", &state_to_save);
-   if (state_to_save < 10 && state_to_save >= 0){
-       FCEUI_SelectState(state_to_save, 0);
-       FCEUI_SaveState(NULL, false);
-   }
+	int state_to_save;
+	g_config->getOption("SDL.AutoSaveState", &state_to_save);
+	if (state_to_save < 10 && state_to_save >= 0){
+	    FCEUI_SelectState(state_to_save, 0);
+	    FCEUI_SaveState(NULL, false);
+	}
 
 	int autoInputPreset;
 	g_config->getOption( "SDL.AutoInputPreset", &autoInputPreset );
@@ -333,6 +508,11 @@ CloseGame(void)
 		saveInputSettingsToFile();
 	}
 
+	if ( tasWin != NULL )
+	{
+		tasWin->requestWindowClose();
+	}
+
 	FCEUI_CloseGame();
 
 	DriverKill();
@@ -359,12 +539,27 @@ int  fceuWrapperSoftReset(void)
 
 int  fceuWrapperHardReset(void)
 {
-	if ( isloaded )
+	if ( isloaded && GameInfo )
 	{
-		std::string lastFile;
-		CloseGame();
-		g_config->getOption ("SDL.LastOpenFile", &lastFile);
-		LoadGame (lastFile.c_str());
+		char romPath[2048];
+
+		romPath[0] = 0;
+
+		if ( GameInfo->archiveFilename )
+		{
+			strcpy( romPath, GameInfo->archiveFilename );
+		}
+		else if ( GameInfo->filename )
+		{
+			strcpy( romPath, GameInfo->filename );
+		}
+
+		if ( romPath[0] != 0 )
+		{
+			CloseGame();
+			//printf("Loading: '%s'\n", romPath );
+			LoadGame ( romPath );
+		}
 	}
 	return 0;
 }
@@ -451,6 +646,7 @@ static const char *DriverUsage =
 static void ShowUsage(const char *prog)
 {
 	int i,j;
+	FCEUD_Message("Starting " FCEU_NAME_AND_VERSION "...\n");
 	printf("\nUsage is as follows:\n%s <options> filename\n\n",prog);
 	puts(DriverUsage);
 #ifdef _S9XLUA_H
@@ -490,11 +686,9 @@ static void ShowUsage(const char *prog)
 	
 }
 
-int  fceuWrapperInit( int argc, char *argv[] )
+// Pre-GUI initialization.
+int  fceuWrapperPreInit( int argc, char *argv[] )
 {
-	int error;
-	std::string s;
-
 	for (int i=0; i<argc; i++)
 	{
 		if ( (strcmp(argv[i], "--help") == 0) || (strcmp(argv[i],"-h") == 0) )
@@ -502,7 +696,24 @@ int  fceuWrapperInit( int argc, char *argv[] )
 			ShowUsage(argv[0]);
 			exit(0);
 		}
+		else if ( strcmp(argv[i], "--no-gui") == 0)
+		{
+			printf("Error: Qt/SDL version does not support --no-gui option.\n");
+			exit(1);
+		}
+		else if ( strcmp(argv[i], "--version") == 0)
+		{
+			printf("%i.%i.%i\n", FCEU_VERSION_MAJOR, FCEU_VERSION_MINOR, FCEU_VERSION_PATCH);
+			exit(0);
+		}
 	}
+	return 0;
+}
+
+int  fceuWrapperInit( int argc, char *argv[] )
+{
+	int opt, error;
+	std::string s;
 
 	FCEUD_Message("Starting " FCEU_NAME_AND_VERSION "...\n");
 
@@ -605,14 +816,35 @@ int  fceuWrapperInit( int argc, char *argv[] )
 		AutoResumePlay = false;
 	}
 
+	// Cheats
+	g_config->getOption ("SDL.CheatsDisabled"     , &globalCheatDisabled);
+	g_config->getOption ("SDL.CheatsDisableAutoLS", &disableAutoLSCheats);
+
+	g_config->getOption ("SDL.DrawInputAids", &drawInputAidsEnable);
+
+	// Initialize Autofire Pattern
+	int autofireOnFrames=1, autofireOffFrames=1;
+	g_config->getOption ("SDL.AutofireOnFrames" , &autofireOnFrames );
+	g_config->getOption ("SDL.AutofireOffFrames", &autofireOffFrames);
+
+	SetAutoFirePattern( autofireOnFrames, autofireOffFrames );
+
 	// check to see if recording HUD to AVI is enabled
 	int rh;
 	g_config->getOption("SDL.RecordHUD", &rh);
-	if( rh == 0)
+	if( rh )
 		FCEUI_SetAviEnableHUDrecording(true);
 	else
 		FCEUI_SetAviEnableHUDrecording(false);
 
+	g_config->getOption("SDL.SuggestReadOnlyReplay"  , &suggestReadOnlyReplay);
+	g_config->getOption("SDL.PauseAfterMoviePlayback", &pauseAfterPlayback);
+	g_config->getOption("SDL.CloseFinishedMovie"     , &closeFinishedMovie);
+	g_config->getOption("SDL.MovieBindSavestate"     , &bindSavestate);
+	g_config->getOption("SDL.SubtitlesOnAVI"         , &subtitlesOnAVI);
+	g_config->getOption("SDL.AutoMovieBackup"        , &autoMovieBackup);
+	g_config->getOption("SDL.MovieFullSaveStateLoads", &fullSaveStateLoads);
+
 	// check to see if movie messages are disabled
 	int mm;
 	g_config->getOption("SDL.MovieMsg", &mm);
@@ -621,6 +853,9 @@ int  fceuWrapperInit( int argc, char *argv[] )
 	else
 		FCEUI_SetAviDisableMovieMessages(false);
   
+	g_config->getOption("SDL.AviVideoFormat", &opt);
+	aviSetSelVideoFormat(opt);
+
 	// check for a .fm2 file to rip the subtitles
 	g_config->getOption("SDL.RipSubs", &s);
 	g_config->setOption("SDL.RipSubs", "");
@@ -653,9 +888,9 @@ int  fceuWrapperInit( int argc, char *argv[] )
 			extern std::vector<std::string> subtitleMessages;
 			float fps = (md.palFlag == 0 ? 60.0988 : 50.0069); // NTSC vs PAL
 			float subduration = 3; // seconds for the subtitles to be displayed
-			for (int i = 0; i < subtitleFrames.size(); i++)
+			for (size_t i = 0; i < subtitleFrames.size(); i++)
 			{
-				fprintf(srtfile, "%i\n", i+1); // starts with 1, not 0
+				fprintf(srtfile, "%zi\n", i+1); // starts with 1, not 0
 				double seconds, ms, endseconds, endms;
 				seconds = subtitleFrames[i]/fps;
 				if (i+1 < subtitleFrames.size()) // there's another subtitle coming after this one
@@ -700,6 +935,8 @@ int  fceuWrapperInit( int argc, char *argv[] )
 	// update the emu core
 	UpdateEMUCore(g_config);
 
+	CalcVideoDimensions();
+
 	#ifdef CREATE_AVI
 	g_config->getOption("SDL.VideoLog", &s);
 	g_config->setOption("SDL.VideoLog", "");
@@ -737,18 +974,33 @@ int  fceuWrapperInit( int argc, char *argv[] )
 
 	if (romIndex >= 0)
 	{
-		// load the specified game
-		error = LoadGame(argv[romIndex]);
-		if (error != 1) 
+		QFileInfo fi( argv[romIndex] );
+
+		// Resolve absolute path to file
+		if ( fi.exists() )
 		{
-			DriverKill();
-			SDL_Quit();
+			std::string fullpath = fi.canonicalFilePath().toStdString().c_str();
+
+			error = LoadGame( fullpath.c_str() );
+
+			if (error != 1)
+			{
+				DriverKill();
+				SDL_Quit();
+				return -1;
+			}
+			g_config->setOption("SDL.LastOpenFile", fullpath.c_str() );
+			g_config->save();
+		}
+		else
+		{
+			// File was not found
 			return -1;
 		}
-		g_config->setOption("SDL.LastOpenFile", argv[romIndex]);
-		g_config->save();
 	}
 
+	aviRecordInit();
+
 	// movie playback
 	g_config->getOption("SDL.Movie", &s);
 	g_config->setOption("SDL.Movie", "");
@@ -793,29 +1045,32 @@ int  fceuWrapperInit( int argc, char *argv[] )
 	// load lua script if option passed
 	g_config->getOption("SDL.LuaScript", &s);
 	g_config->setOption("SDL.LuaScript", "");
-	if (s != "")
+	if (s.size() > 0)
 	{
-#if defined(__linux) || defined(__APPLE__)
+		QFileInfo fi( s.c_str() );
 
 		// Resolve absolute path to file
-		char fullpath[2048];
-		if ( realpath( s.c_str(), fullpath ) != NULL )
+		if ( fi.exists() )
 		{
-			//printf("Fullpath: '%s'\n", fullpath );
-			s.assign( fullpath );
+			//printf("FI: '%s'\n", fi.absoluteFilePath().toStdString().c_str() );
+			//printf("FI: '%s'\n", fi.canonicalFilePath().toStdString().c_str() );
+			s = fi.canonicalFilePath().toStdString();
 		}
-#endif
+//#if defined(__linux__) || defined(__APPLE__) || defined(__unix__)
+//
+//		// Resolve absolute path to file
+//		char fullpath[2048];
+//		if ( realpath( s.c_str(), fullpath ) != NULL )
+//		{
+//			printf("Fullpath: '%s'\n", fullpath );
+//			s.assign( fullpath );
+//		}
+//#endif
 		FCEU_LoadLuaCode(s.c_str());
 	}
 #endif
 	
-	{
-		int id;
-		g_config->getOption("SDL.NewPPU", &id);
-		if (id)
-			newppu = 1;
-	}
-
+	g_config->getOption("SDL.NewPPU", &newppu);
 	g_config->getOption("SDL.Frameskip", &frameskip);
 
 	return 0;
@@ -832,131 +1087,148 @@ int  fceuWrapperClose( void )
 	return 0;
 }
 
+int  fceuWrapperMemoryCleanup(void)
+{
+	FreeCDLog();
+
+	close_nes_shm();
+
+	if ( g_config )
+	{
+		delete g_config; g_config = NULL;
+	}
+
+	return 0;
+}
+
 /**
  * Update the video, audio, and input subsystems with the provided
  * video (XBuf) and audio (Buffer) information.
  */
 void
 FCEUD_Update(uint8 *XBuf,
-			 int32 *Buffer,
-			 int Count)
+	 int32 *Buffer,
+	 int Count)
 {
 	int blitDone = 0;
-	extern int FCEUDnetplay;
+	//extern int FCEUDnetplay;
 
-	#ifdef CREATE_AVI
-	if (LoggingEnabled == 2 || (eoptions&EO_NOTHROTTLE))
-	{
-		if(LoggingEnabled == 2)
-		{
-			int16* MonoBuf = new int16[Count];
-			int n;
-			for(n=0; n<Count; ++n)
-			{
-				MonoBuf[n] = Buffer[n] & 0xFFFF;
-			}
-			NESVideoLoggingAudio
-			(
-			  MonoBuf, 
-			  FSettings.SndRate, 16, 1,
-			  Count
-			);
-			delete [] MonoBuf;
-		}
-		Count /= 2;
-		if (inited & 1)
-		{
-			if (Count > GetWriteSound()) Count = GetWriteSound();
+	//#ifdef CREATE_AVI
+	//if (LoggingEnabled == 2 || (eoptions&EO_NOTHROTTLE))
+	//{
+	//	if(LoggingEnabled == 2)
+	//	{
+	//		int16* MonoBuf = new int16[Count];
+	//		int n;
+	//		for(n=0; n<Count; ++n)
+	//		{
+	//			MonoBuf[n] = Buffer[n] & 0xFFFF;
+	//		}
+	//		NESVideoLoggingAudio
+	//		(
+	//		  MonoBuf, 
+	//		  FSettings.SndRate, 16, 1,
+	//		  Count
+	//		);
+	//		delete [] MonoBuf;
+	//	}
+	//	Count /= 2;
+	//	if (inited & 1)
+	//	{
+	//		if (Count > GetWriteSound()) Count = GetWriteSound();
 
-			if (!mutecapture)
-			{
-				if(Count > 0 && Buffer) WriteSound(Buffer,Count);   
-			}
-		}
-		//if (inited & 2)
-		//	FCEUD_UpdateInput();
-	  	if(XBuf && (inited & 4)) BlitScreen(XBuf);
-	  
-		return;
-	}
-	#endif
+	//		if (!mutecapture)
+	//		{
+	//			if(Count > 0 && Buffer) WriteSound(Buffer,Count);   
+	//		}
+	//	}
+	//	//if (inited & 2)
+	//	//	FCEUD_UpdateInput();
+	//  	if(XBuf && (inited & 4)) BlitScreen(XBuf);
+	//  
+	//	return;
+	//}
+	//#endif
+	aviRecordAddAudioFrame( Buffer, Count );
 	
-	int ocount = Count;
+	WriteSound(Buffer,Count);
+
+	//int ocount = Count;
 	// apply frame scaling to Count
-	Count = (int)(Count / g_fpsScale);
-	if (Count) 
-	{
-		int32 can=GetWriteSound();
-		static int uflow=0;
-		int32 tmpcan;
+	//Count = (int)(Count / g_fpsScale);
+	//if (Count) 
+	//{
+	//	int32 can=GetWriteSound();
+	//	static int uflow=0;
+	//	int32 tmpcan;
 
-		// don't underflow when scaling fps
-		if(can >= GetMaxSound() && g_fpsScale==1.0) uflow=1;	/* Go into massive underflow mode. */
+	//	// don't underflow when scaling fps
+	//	if(can >= GetMaxSound() && g_fpsScale==1.0) uflow=1;	/* Go into massive underflow mode. */
 
-		if(can > Count) can=Count;
-		else uflow=0;
+	//	if(can > Count) can=Count;
+	//	else uflow=0;
 
-		#ifdef CREATE_AVI
-		if (!mutecapture)
-		#endif
-		  WriteSound(Buffer,can);
+	//	#ifdef CREATE_AVI
+	//	if (!mutecapture)
+	//	#endif
+	//	  WriteSound(Buffer,can);
 
-		//if(uflow) puts("Underflow");
-		tmpcan = GetWriteSound();
-		// don't underflow when scaling fps
-		if (g_fpsScale>1.0 || ((tmpcan < Count*0.90) && !uflow)) 
-		{
-			if (XBuf && (inited&4) && !(NoWaiting & 2))
-			{
-				BlitScreen(XBuf); blitDone = 1;
-			}
-			Buffer+=can;
-			Count-=can;
-			if(Count) 
-			{
-				if(NoWaiting) 
-				{
-					can=GetWriteSound();
-					if(Count>can) Count=can;
-					#ifdef CREATE_AVI
-					if (!mutecapture)
-					#endif
-					  WriteSound(Buffer,Count);
-				}
-			  	else
-			  	{
-					while(Count>0) 
-					{
-						#ifdef CREATE_AVI
-						if (!mutecapture)
-						#endif
-						  WriteSound(Buffer,(Count<ocount) ? Count : ocount);
-						Count -= ocount;
-					}
-				}
-			}
-		} //else puts("Skipped");
-		else if (!NoWaiting && FCEUDnetplay && (uflow || tmpcan >= (Count * 1.8))) 
-		{
-			if (Count > tmpcan) Count=tmpcan;
-			while(tmpcan > 0) 
-			{
-				//	printf("Overwrite: %d\n", (Count <= tmpcan)?Count : tmpcan);
-				#ifdef CREATE_AVI
-				if (!mutecapture)
-				#endif
-				  WriteSound(Buffer, (Count <= tmpcan)?Count : tmpcan);
-				tmpcan -= Count;
-			}
-		}
-	}
-  	else 
-	{
-		if (XBuf && (inited&4)) 
-		{
-			BlitScreen(XBuf); blitDone = 1;
-		}
-	}
+	//	//if(uflow) puts("Underflow");
+	//	tmpcan = GetWriteSound();
+	//	// don't underflow when scaling fps
+	//	if (g_fpsScale>1.0 || ((tmpcan < Count*0.90) && !uflow)) 
+	//	{
+	//		if (XBuf && (inited&4) && !(NoWaiting & 2))
+	//		{
+	//			BlitScreen(XBuf); blitDone = 1;
+	//		}
+	//		Buffer+=can;
+	//		Count-=can;
+	//		if(Count) 
+	//		{
+	//			if(NoWaiting) 
+	//			{
+	//				can=GetWriteSound();
+	//				if(Count>can) Count=can;
+	//				#ifdef CREATE_AVI
+	//				if (!mutecapture)
+	//				#endif
+	//				  WriteSound(Buffer,Count);
+	//			}
+	//		  	else
+	//		  	{
+	//				while(Count>0) 
+	//				{
+	//					#ifdef CREATE_AVI
+	//					if (!mutecapture)
+	//					#endif
+	//					  WriteSound(Buffer,(Count<ocount) ? Count : ocount);
+	//					Count -= ocount;
+	//				}
+	//			}
+	//		}
+	//	} //else puts("Skipped");
+	//	//else if (!NoWaiting && FCEUDnetplay && (uflow || tmpcan >= (Count * 1.8))) 
+	//	//{
+	//	//	if (Count > tmpcan) Count=tmpcan;
+	//	//	while(tmpcan > 0) 
+	//	//	{
+	//	//		//	printf("Overwrite: %d\n", (Count <= tmpcan)?Count : tmpcan);
+	//	//		#ifdef CREATE_AVI
+	//	//		if (!mutecapture)
+	//	//		#endif
+	//	//		  WriteSound(Buffer, (Count <= tmpcan)?Count : tmpcan);
+	//	//		tmpcan -= Count;
+	//	//	}
+	//	//}
+	//}
+  	//else 
+	//{
+	//	if (XBuf && (inited&4)) 
+	//	{
+	//		BlitScreen(XBuf); blitDone = 1;
+	//	}
+	//}
 	if ( !blitDone )
 	{
 		if (XBuf && (inited&4)) 
@@ -969,12 +1241,29 @@ FCEUD_Update(uint8 *XBuf,
 
 static void DoFun(int frameskip, int periodic_saves)
 {
-	uint8 *gfx;
-	int32 *sound;
-	int32 ssize;
+	uint8 *gfx = 0;
+	int32 *sound = 0;
+	int32 ssize = 0;
 	static int fskipc = 0;
 	//static int opause = 0;
 
+	// If TAS editor is engaged, check whether a seek frame is set.
+	// If a seek is in progress, don't emulate past target frame.
+	if ( tasWindowIsOpen() )
+	{
+		int runToFrameTarget;
+	
+		runToFrameTarget = PLAYBACK::getPauseFrame();
+
+		if ( runToFrameTarget >= 0)
+		{
+			if ( currFrameCounter >= runToFrameTarget )
+			{
+				FCEUI_SetEmulationPaused(EMULATIONPAUSED_PAUSED);
+				return;
+			}
+		}
+	}
     //TODO peroidic saves, working on it right now
     if (periodic_saves && FCEUD_GetTime() % PERIODIC_SAVE_INTERVAL < 30){
         FCEUI_SaveState(NULL, false);
@@ -983,7 +1272,7 @@ static void DoFun(int frameskip, int periodic_saves)
 	fskipc = (fskipc + 1) % (frameskip + 1);
 #endif
 
-	if (NoWaiting) 
+	if (NoWaiting || turbo) 
 	{
 		gfx = 0;
 	}
@@ -999,6 +1288,30 @@ static void DoFun(int frameskip, int periodic_saves)
 	emulatorCycleCount++;
 }
 
+static std::string lockFile;
+static bool debugMutexLock = false;
+
+void fceuWrapperLock(const char *filename, int line, const char *func)
+{
+	fceuWrapperLock();
+
+	if ( debugMutexLock )
+	{
+		char txt[32];
+
+		if ( mutexLocks > 1 )
+		{
+			printf("Recursive Lock:%i\n", mutexLocks );
+			printf("Already Locked By: %s\n", lockFile.c_str() );
+			printf("Requested By: %s:%i - %s\n", filename, line, func );
+		}
+		sprintf( txt, ":%i - ", line );
+		lockFile.assign(filename);
+		lockFile.append(txt);
+		lockFile.append(func);
+	}
+}
+
 void fceuWrapperLock(void)
 {
 	mutexPending++;
@@ -1010,6 +1323,23 @@ void fceuWrapperLock(void)
 	mutexLocks++;
 }
 
+bool fceuWrapperTryLock(const char *filename, int line, const char *func, int timeout)
+{
+	bool lockAcq = false;
+
+	lockAcq = fceuWrapperTryLock( timeout );
+
+	if ( lockAcq && debugMutexLock)
+	{
+		char txt[32];
+		sprintf( txt, ":%i - ", line );
+		lockFile.assign(filename);
+		lockFile.append(txt);
+		lockFile.append(func);
+	}
+	return lockAcq;
+}
+
 bool fceuWrapperTryLock(int timeout)
 {
 	bool lockAcq = false;
@@ -1032,15 +1362,16 @@ void fceuWrapperUnLock(void)
 {
 	if ( mutexLocks > 0 )
 	{
+		mutexLocks--;
 		if ( consoleWindow != NULL )
 		{
 			consoleWindow->mutex->unlock();
 		}
-		mutexLocks--;
 	}
 	else
 	{
 		printf("Error: Mutex is Already UnLocked\n");
+		//abort(); // Uncomment to catch a stack trace
 	}
 }
 
@@ -1052,27 +1383,33 @@ bool fceuWrapperIsLocked(void)
 int  fceuWrapperUpdate( void )
 {
 	bool lock_acq;
+	static bool mutexLockFail = false;
 
 	// If a request is pending, 
 	// sleep to allow request to be serviced.
 	if ( mutexPending > 0 )
 	{
-		usleep( 100000 );
+		msleep( 16 );
 	}
 
-	lock_acq = fceuWrapperTryLock();
+	lock_acq = fceuWrapperTryLock( __FILE__, __LINE__, __func__ );
 
 	if ( !lock_acq )
 	{
 		if ( GameInfo )
 		{
-			printf("Error: Emulator Failed to Acquire Mutex\n");
+			if ( !mutexLockFail )
+			{
+				printf("Warning: Emulator Thread Failed to Acquire Mutex - GUI has Lock\n");
+			}
+			mutexLockFail = true;
 		}
-		usleep( 100000 );
+		msleep( 16 );
 
 		return -1;
 	}
-	emulatorHasMutux = 1;
+	mutexLockFail = false;
+	emulatorHasMutex = 1;
  
 	if ( GameInfo )
 	{
@@ -1080,9 +1417,13 @@ int  fceuWrapperUpdate( void )
 	
 		hexEditorUpdateMemoryValues();
 
+		if ( consoleWindow )
+		{
+			consoleWindow->emulatorThread->signalFrameFinished();
+		}
 		fceuWrapperUnLock();
 
-		emulatorHasMutux = 0;
+		emulatorHasMutex = 0;
 
 		while ( SpeedThrottle() )
 		{
@@ -1095,9 +1436,9 @@ int  fceuWrapperUpdate( void )
 	{
 		fceuWrapperUnLock();
 
-		emulatorHasMutux = 0;
+		emulatorHasMutex = 0;
 
-		usleep( 100000 );
+		msleep( 100 );
 	}
 	return 0;
 }
@@ -1169,7 +1510,7 @@ FCEUFILE* FCEUD_OpenArchive(ArchiveScanRecord& asr, std::string& fname, std::str
 
 		for (size_t i=0; i<asr.files.size(); i++)
 		{
-			char base[512], suffix[32];
+			char base[512], suffix[128];
 
 			getFileBaseName( asr.files[i].name.c_str(), base, suffix );
 
@@ -1382,16 +1723,28 @@ FCEUFILE* FCEUD_OpenArchiveIndex(ArchiveScanRecord& asr, std::string &fname, int
     }
 DUMMY(FCEUD_HideMenuToggle)
 DUMMY(FCEUD_MovieReplayFrom)
-DUMMY(FCEUD_ToggleStatusIcon)
-DUMMY(FCEUD_AviRecordTo)
-DUMMY(FCEUD_AviStop)
-void FCEUI_AviVideoUpdate(const unsigned char* buffer) { }
-int FCEUD_ShowStatusIcon(void) {return 0;}
-bool FCEUI_AviIsRecording(void) {return false;}
+//DUMMY(FCEUD_AviRecordTo)
+//DUMMY(FCEUD_AviStop)
+//void FCEUI_AviVideoUpdate(const unsigned char* buffer) { }
+//bool FCEUI_AviIsRecording(void) {return false;}
 void FCEUI_UseInputPreset(int preset) { }
 bool FCEUD_PauseAfterPlayback() { return pauseAfterPlayback; }
 
-void FCEUD_TurboOn	 (void) { /* TODO */ };
-void FCEUD_TurboOff   (void) { /* TODO */ };
-void FCEUD_TurboToggle(void) { /* TODO */ };
+int FCEUD_ShowStatusIcon(void)
+{
+	return showStatusIconOpt;
+}
+void FCEUD_ToggleStatusIcon(void)
+{
+	showStatusIconOpt = !showStatusIconOpt;
+}
+
+bool FCEUD_ShouldDrawInputAids(void)
+{
+	return drawInputAidsEnable;
+}
+
+void FCEUD_TurboOn (void) { turbo = true; };
+void FCEUD_TurboOff   (void) { turbo = false; };
+void FCEUD_TurboToggle(void) { turbo = !turbo; };
 
