@@ -1,4 +1,23 @@
-// HotKeyConf.cpp
+/* FCE Ultra - NES/Famicom Emulator
+ *
+ * Copyright notice for this file:
+ *  Copyright (C) 2020 mjbudd77
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+// iNesHeaderEditor.cpp
 //
 #include <stdio.h>
 #include <stdlib.h>
@@ -10,6 +29,7 @@
 #include <QCloseEvent>
 #include <QMessageBox>
 #include <QFileDialog>
+#include <QSettings>
 
 #include "../../types.h"
 #include "../../fceu.h"
@@ -130,6 +150,12 @@ static const char* inputDevList[] = {
 	"RacerMate Bicycle",
 	"U-Force",
 	"R.O.B. Stack-Up",
+	"City Patrolman Lightgun",
+	"Sharp C1 Cassette Interface",
+	"Standard Controller with swapped Left-Right/Up-Down/B-A",
+	"Excalibor Sudoku Pad",
+	"ABL Pinball",
+	"Golden Nugget Casino extra buttons",
 	0
 };
 //----------------------------------------------------------------------------
@@ -144,8 +170,12 @@ iNesHeaderEditor_t::iNesHeaderEditor_t(QWidget *parent)
 	QHBoxLayout *hbox, *hbox1, *hbox2, *hbox3;
 	QGroupBox *box, *hdrBox;
 	QGridLayout *grid;
+	QStyle      *style;
+	QSettings    settings;
 	char stmp[128];
 
+	style = this->style();
+
 	initOK = false;
 
 	font.setFamily("Courier New");
@@ -159,7 +189,7 @@ iNesHeaderEditor_t::iNesHeaderEditor_t(QWidget *parent)
     fontCharWidth = fm.width(QLatin1Char('2'));
 #endif
 
-	setWindowTitle("iNES Header Editor");
+	setWindowTitle("NES Header Editor");
 
 	//resize( 512, 512 );
 
@@ -167,7 +197,7 @@ iNesHeaderEditor_t::iNesHeaderEditor_t(QWidget *parent)
 	hdrLayout  = new QVBoxLayout();
 	hbox1      = new QHBoxLayout();
 	hbox       = new QHBoxLayout();
-	hdrBox     = new QGroupBox( tr("iNES Header") );
+	hdrBox     = new QGroupBox( tr("NES Header") );
 	box        = new QGroupBox( tr("Version:") );
 
 	mainLayout->addWidget( hdrBox );
@@ -210,7 +240,7 @@ iNesHeaderEditor_t::iNesHeaderEditor_t(QWidget *parent)
 	{
 		sprintf(stmp, "%d %s", bmap[i].number, bmap[i].name);
 		
-		mapperComboBox->addItem( tr(stmp), i );
+		mapperComboBox->addItem( tr(stmp), bmap[i].number );
 	}
 	hdrLayout->addLayout( hbox1 );
 
@@ -435,6 +465,7 @@ iNesHeaderEditor_t::iNesHeaderEditor_t(QWidget *parent)
 	miscRomsEdit->setMaxLength( 1 );
 	miscRomsEdit->setValidator( validator );
 	miscRomsEdit->setAlignment(Qt::AlignCenter);
+	miscRomsEdit->setMinimumWidth( 3 * fontCharWidth );
 	miscRomsEdit->setMaximumWidth( 3 * fontCharWidth );
 
 	grid = new QGridLayout();
@@ -442,6 +473,10 @@ iNesHeaderEditor_t::iNesHeaderEditor_t(QWidget *parent)
 	saveAsBtn  = new QPushButton( tr("Save As") );
 	closeBtn   = new QPushButton( tr("Close") );
 
+	restoreBtn->setIcon( style->standardIcon( QStyle::SP_DialogResetButton ) );
+	saveAsBtn->setIcon( style->standardIcon( QStyle::SP_DialogSaveButton ) );
+	closeBtn->setIcon( style->standardIcon( QStyle::SP_DialogCloseButton ) );
+
 	grid->addWidget( restoreBtn, 0, 0, Qt::AlignLeft );
 	grid->addWidget( saveAsBtn , 0, 1, Qt::AlignRight );
 	grid->addWidget( closeBtn  , 0, 2, Qt::AlignRight );
@@ -497,24 +532,28 @@ iNesHeaderEditor_t::iNesHeaderEditor_t(QWidget *parent)
 	setHeaderData( iNesHdr );
 
 	initOK = true;
+
+	restoreGeometry(settings.value("iNesHeaderWindow/geometry").toByteArray());
 }
 //----------------------------------------------------------------------------
 iNesHeaderEditor_t::~iNesHeaderEditor_t(void)
 {
-	printf("Destroy Header Editor Config Window\n");
+	QSettings settings;
+	//printf("Destroy Header Editor Config Window\n");
 
 	if ( iNesHdr )
 	{
 		free( iNesHdr ); iNesHdr = NULL;
 	}
+	settings.setValue("iNesHeaderWindow/geometry", saveGeometry());
 }
 //----------------------------------------------------------------------------
 void iNesHeaderEditor_t::closeEvent(QCloseEvent *event)
 {
-   printf("iNES Header Editor Close Window Event\n");
-   done(0);
+	//printf("NES Header Editor Close Window Event\n");
+	done(0);
 	deleteLater();
-   event->accept();
+	event->accept();
 }
 //----------------------------------------------------------------------------
 void iNesHeaderEditor_t::closeWindow(void)
@@ -656,27 +695,27 @@ bool iNesHeaderEditor_t::loadHeader(iNES_HEADER* header)
 		{
 			case errors::OPEN_FAILED:
 			{
-				char buf[2200];
-				sprintf(buf, "Error opening %s!", LoadedRomFName);
+				char buf[5120];
+				snprintf(buf, sizeof(buf), "Error opening %s!", LoadedRomFName);
 				showErrorMsgWindow( buf );
 				break;
 			}
 			case errors::INVALID_HEADER:
-				//MessageBox(parent, "Invalid iNES header.", "iNES Header Editor", MB_OK | MB_ICONERROR);
-				showErrorMsgWindow( "Invalid iNES header." );
+				//MessageBox(parent, "Invalid NES header.", "NES Header Editor", MB_OK | MB_ICONERROR);
+				showErrorMsgWindow( "Invalid NES header." );
 				break;
 			case errors::FDS_HEADER:
-				//MessageBox(parent, "Editing header of an FDS file is not supported.", "iNES Header Editor", MB_OK | MB_ICONERROR);
+				//MessageBox(parent, "Editing header of an FDS file is not supported.", "NES Header Editor", MB_OK | MB_ICONERROR);
 				showErrorMsgWindow("Editing header of an FDS file is not supported.");
 				break;
 			case errors::UNIF_HEADER:
-				//MessageBox(parent, "Editing header of a UNIF file is not supported.", "iNES Header Editor", MB_OK | MB_ICONERROR);
+				//MessageBox(parent, "Editing header of a UNIF file is not supported.", "NES Header Editor", MB_OK | MB_ICONERROR);
 				showErrorMsgWindow("Editing header of a UNIF file is not supported.");
 				break;
 			case errors::NSF_HEADER:
 //			case errors::NSF2_HEADER:
 //			case errors::NSFE_HEADER:
-				//MessageBox(parent, "Editing header of an NSF file is not supported.", "iNES Header Editor", MB_OK | MB_ICONERROR);
+				//MessageBox(parent, "Editing header of an NSF file is not supported.", "NES Header Editor", MB_OK | MB_ICONERROR);
 				showErrorMsgWindow("Editing header of an NSF file is not supported.");
 				break;
 		}
@@ -698,8 +737,9 @@ bool iNesHeaderEditor_t::SaveINESFile(const char* path, iNES_HEADER* header)
 	FCEUFILE* source = FCEU_fopen(LoadedRomFName, NULL, "rb", 0, -1, ext);
 	if (!source)
 	{
-		sprintf(buf, "Opening source file %s failed.", LoadedRomFName);
-		showErrorMsgWindow(buf);
+		char msg[5120];
+		snprintf(msg, sizeof(msg), "Opening source file %s failed.", LoadedRomFName);
+		showErrorMsgWindow(msg);
 		return false;
 	}
 
@@ -707,8 +747,9 @@ bool iNesHeaderEditor_t::SaveINESFile(const char* path, iNES_HEADER* header)
 	FILE* target = FCEUD_UTF8fopen(path, "wb");
 	if (!target)
 	{
-		sprintf(buf, "Creating target file %s failed.", path);
-		showErrorMsgWindow(buf);
+		char msg[5120];
+		snprintf(msg, sizeof(msg), "Creating target file %s failed.", path);
+		showErrorMsgWindow(msg);
 		return false;
 	}
 
@@ -737,7 +778,7 @@ bool iNesHeaderEditor_t::openFile(void)
 	int ret, useNativeFileDialogVal;
 	QString filename;
 	std::string last;
-	char dir[512];
+	std::string dir;
 	QFileDialog  dialog(this, tr("Open NES File") );
 
 	const QStringList filters(
@@ -758,14 +799,13 @@ bool iNesHeaderEditor_t::openFile(void)
 
 	getDirFromFile( last.c_str(), dir );
 
-	dialog.setDirectory( tr(dir) );
+	dialog.setDirectory( tr(dir.c_str()) );
 
 	// Check config option to use native file dialog or not
 	g_config->getOption ("SDL.UseNativeFileDialog", &useNativeFileDialogVal);
 
 	dialog.setOption(QFileDialog::DontUseNativeDialog, !useNativeFileDialogVal);
 
-	dialog.show();
 	ret = dialog.exec();
 
 	if ( ret )
@@ -787,7 +827,8 @@ bool iNesHeaderEditor_t::openFile(void)
 
 	if ( GameInfo == NULL )
 	{
-		strcpy( LoadedRomFName, filename.toStdString().c_str() );
+		strncpy( LoadedRomFName, filename.toStdString().c_str(), sizeof(LoadedRomFName)-1 );
+		LoadedRomFName[sizeof(LoadedRomFName)-1] = 0;
 	}
 
    return true;
@@ -798,7 +839,7 @@ void iNesHeaderEditor_t::saveFileAs(void)
 	int ret, useNativeFileDialogVal;
 	QString filename;
 	std::string last;
-	char dir[512];
+	std::string dir;
 	QFileDialog  dialog(this, tr("Save iNES File") );
 
 	dialog.setFileMode(QFileDialog::AnyFile);
@@ -812,14 +853,13 @@ void iNesHeaderEditor_t::saveFileAs(void)
 
 	getDirFromFile( LoadedRomFName, dir );
 
-	dialog.setDirectory( tr(dir) );
+	dialog.setDirectory( tr(dir.c_str()) );
 
 	// Check config option to use native file dialog or not
 	g_config->getOption ("SDL.UseNativeFileDialog", &useNativeFileDialogVal);
 
 	dialog.setOption(QFileDialog::DontUseNativeDialog, !useNativeFileDialogVal);
 
-	dialog.show();
 	ret = dialog.exec();
 
 	if ( ret )
@@ -867,7 +907,21 @@ void iNesHeaderEditor_t::setHeaderData(iNES_HEADER* header)
 	{
 		mapper |= (header->ROM_type3 & 0x0F) << 8;
 	}
-	mapperComboBox->setCurrentIndex( mapper );
+	for (i=0; i<mapperComboBox->count(); i++)
+	{
+		if ( mapperComboBox->itemData(i).toInt() > mapper )
+		{
+			sprintf( buf, "%i Unknown/unsupported", i);
+			mapperComboBox->insertItem( i, tr(buf), mapper);
+			mapperComboBox->setCurrentIndex(i);
+			break;
+		}
+		else if ( mapperComboBox->itemData(i).toInt() == mapper )
+		{
+			mapperComboBox->setCurrentIndex(i);
+			break;
+		}
+	}
 
 	// Sub Mapper
 	sprintf(buf, "%d", ines20 ? header->ROM_type3 >> 4 : 0);
@@ -1146,7 +1200,7 @@ void iNesHeaderEditor_t::setHeaderData(iNES_HEADER* header)
 	}
 
 	// Input Device:
-	int input = header->reserved[1] & 0x1F;
+	int input = header->expansion & 0x3F;
 	for (i=0; i<inputDevBox->count(); i++)
 	{
 		if ( inputDevBox->itemData(i).toInt() == input )
@@ -1156,7 +1210,7 @@ void iNesHeaderEditor_t::setHeaderData(iNES_HEADER* header)
 	}
 
 	// Miscellaneous ROM Area(s)
-	sprintf(buf, "%d", header->reserved[0] & 3);
+	sprintf(buf, "%d", header->misc_roms & 3);
 	miscRomsEdit->setText( tr(buf) );
 
 	// Trainer
@@ -1869,7 +1923,7 @@ bool iNesHeaderEditor_t::WriteHeaderData(iNES_HEADER* header)
 		int input = inputDevBox->itemData(idx).toInt();
 		if (input <= 0x3F)
 		{
-			_header.reserved[1] |= input & 0x3F;
+			_header.expansion |= input & 0x3F;
 		}
 		else
 		{
@@ -1885,15 +1939,15 @@ bool iNesHeaderEditor_t::WriteHeaderData(iNES_HEADER* header)
 		int misc_roms = 0;
 		if (sscanf(buf, "%d", &misc_roms) < 1)
 		{
-			showErrorMsgWindow("Invalid miscellanous ROM(s) count. If you don't know what value should be, we recommend to set it to 0.");
+			showErrorMsgWindow("Invalid miscellaneous ROM(s) count. If you don't know what value should be, we recommend to set it to 0.");
 			return false;
 		}
 		if (misc_roms > 3)
 		{
-			showErrorMsgWindow("Miscellanous ROM(s) count has exceeded the limit of iNES 2.0 (3)");
+			showErrorMsgWindow("Miscellaneous ROM(s) count has exceeded the limit of iNES 2.0 (3)");
 			return false;
 		}
-		_header.reserved[0] |= misc_roms & 3;
+		_header.misc_roms |= misc_roms & 3;
 	}
 
 	// iNES 1.0 unofficial properties
@@ -1971,7 +2025,7 @@ void iNesHeaderEditor_t::printHeader(iNES_HEADER* _header)
 	printf("%02X ", _header->VRAM_size);
 	printf("%02X ", _header->TV_system);
 	printf("%02X ", _header->VS_hardware);
-	printf("%02X ", _header->reserved[0]);
-	printf("%02X\n", _header->reserved[1]);
+	printf("%02X ", _header->misc_roms);
+	printf("%02X\n", _header->expansion);
 }
 //----------------------------------------------------------------------------
