@@ -65,9 +65,9 @@ static float  *moire   = NULL;	// modulated signal
 const  float   phasex  = (float) 5/18*2;
 const  float   phasey  = (float) 1/ 6*2;
 const  float   pi      = 3.14f;
-int    palnotch        = 90;
+int    palnotch        = 100;
 int    palsaturation   = 100;
-int    palsharpness    = 50;
+int    palsharpness    = 0;
 int    palcontrast     = 100;
 int    palbrightness   = 50;
 bool   palupdate       = 1;
@@ -107,8 +107,6 @@ static void CalculateShift(uint32 *CBM, int *cshiftr, int *cshiftl)
 
 int InitBlitToHigh(int b, uint32 rmask, uint32 gmask, uint32 bmask, int efx, int specfilt, int specfilteropt)
 {
-	//paldeemphswap = 0; // determine this in FCEUPPU_SetVideoSystem() instead
-
 	// -Video Modes Tag-
 	if(specfilt == 3) // NTSC 2x
 	{
@@ -208,6 +206,7 @@ int InitBlitToHigh(int b, uint32 rmask, uint32 gmask, uint32 bmask, int efx, int
 		palrgb     = (uint32 *)FCEU_dmalloc((256+512)*PAL_PHASES*sizeof(uint32));
 		palrgb2    = (uint32 *)FCEU_dmalloc((256+512)*PAL_PHASES*sizeof(uint32));
 		moire      = (float  *)FCEU_dmalloc(          PAL_PHASES*sizeof(float));
+		palupdate  = 1;
 	}
 
 	silt = specfilt;	
@@ -239,18 +238,18 @@ void KillBlitToHigh(void)
 {
 	if(palettetranslate)
 	{
-		free(palettetranslate);
+		FCEU_free(palettetranslate);
 		palettetranslate=NULL;
 	}
 	
 	if(specbuf8bpp)
 	{
-		free(specbuf8bpp);
+		FCEU_free(specbuf8bpp);
 		specbuf8bpp = NULL;
 	}
 	if(specbuf32bpp)
 	{
-		free(specbuf32bpp);
+		FCEU_free(specbuf32bpp);
 		specbuf32bpp = NULL;
 	}
 	if(specbuf)
@@ -260,10 +259,11 @@ void KillBlitToHigh(void)
 			hq3x_Kill();
 		else
 			hq2x_Kill();
+		FCEU_free(specbuf);
 		specbuf=NULL;
 	}
 	if (nes_ntsc) {
-		free(nes_ntsc);
+		FCEU_free(nes_ntsc);
 		nes_ntsc = NULL;
 	}
 	if (ntscblit) {
@@ -466,7 +466,7 @@ void Blit8To8(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale, in
 /* Todo:  Make sure 24bpp code works right with big-endian cpus */
 
 //takes a pointer to XBuf and applies fully modern deemph palettizing
-u32 ModernDeemphColorMap(u8* src, u8* srcbuf, int xscale, int yscale)
+template<int SCALE> static u32 _ModernDeemphColorMap(const u8* src, const u8* srcbuf)
 {
 	u8 pixel = *src;
 	
@@ -476,8 +476,8 @@ u32 ModernDeemphColorMap(u8* src, u8* srcbuf, int xscale, int yscale)
 	int ofs = src-srcbuf;
 	int xofs = ofs&255;
 	int yofs = ofs>>8;
-	if(xscale!=1) xofs /= xscale; //untested optimization
-	if(yscale!=1) yofs /= yscale; //untested optimization
+	xofs /= SCALE;
+	yofs /= SCALE;
 	ofs = xofs+yofs*256;
 
 	//find out which deemph bitplane value we're on
@@ -485,11 +485,47 @@ u32 ModernDeemphColorMap(u8* src, u8* srcbuf, int xscale, int yscale)
 
 	//if it was a deemph'd value, grab it from the deemph palette
 	if(deemph != 0)
-		color = palettetranslate[256+(pixel&0x3F)+deemph*64];
+	{
+		color = palettetranslate[256+(pixel&0x3F)+(deemph*64)];
+	}
 
 	return color;
 }
 
+u32 ModernDeemphColorMap(const u8* src, const u8* srcbuf, int scale)
+{
+	if(scale == 1) return _ModernDeemphColorMap<1>(src,srcbuf);
+	else if(scale == 2) return _ModernDeemphColorMap<2>(src,srcbuf);
+	else if(scale == 3) return _ModernDeemphColorMap<3>(src,srcbuf);
+	else if(scale == 4) return _ModernDeemphColorMap<4>(src,srcbuf);
+	else if(scale == 5) return _ModernDeemphColorMap<5>(src,srcbuf);
+	else if(scale == 6) return _ModernDeemphColorMap<6>(src,srcbuf);
+	else if(scale == 7) return _ModernDeemphColorMap<7>(src,srcbuf);
+	else if(scale == 8) return _ModernDeemphColorMap<8>(src,srcbuf);
+	else if(scale == 9) return _ModernDeemphColorMap<9>(src,srcbuf);
+	else { FCEU_abort("unhandled ModernDeemphColorMap scale"); return 0; }
+}
+
+typedef u32 (*ModernDeemphColorMapFuncPtr)( const u8*, const u8* );
+
+static ModernDeemphColorMapFuncPtr getModernDeemphColorMapFunc(int scale)
+{
+	ModernDeemphColorMapFuncPtr ptr;
+
+	if(scale == 1) ptr = &_ModernDeemphColorMap<1>;
+	else if(scale == 2) ptr = &_ModernDeemphColorMap<2>;
+	else if(scale == 3) ptr = &_ModernDeemphColorMap<3>;
+	else if(scale == 4) ptr = &_ModernDeemphColorMap<4>;
+	else if(scale == 5) ptr = &_ModernDeemphColorMap<5>;
+	else if(scale == 6) ptr = &_ModernDeemphColorMap<6>;
+	else if(scale == 7) ptr = &_ModernDeemphColorMap<7>;
+	else if(scale == 8) ptr = &_ModernDeemphColorMap<8>;
+	else if(scale == 9) ptr = &_ModernDeemphColorMap<9>;
+	else { FCEU_abort("unhandled ModernDeemphColorMap scale"); ptr = nullptr; }
+
+	return ptr;
+}
+
 void Blit8ToHigh(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale, int yscale)
 {
 	int x,y;
@@ -505,6 +541,7 @@ void Blit8ToHigh(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale,
 	{
 		int mult; 
 		int base;
+		ModernDeemphColorMapFuncPtr ModernDeemphColorMapFunc = NULL;
 		
 		// -Video Modes Tag-
 		if(silt == 2) mult = 2;
@@ -513,7 +550,12 @@ void Blit8ToHigh(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale,
 		Blit8To8(src, specbuf8bpp, xr, yr, 256*mult, xscale, yscale, 0, silt);
 		int mdcmxs = xscale*mult;
 		int mdcmys = yscale*mult;
+
+		if(mdcmxs != mdcmys)
+			abort();
 		
+		ModernDeemphColorMapFunc = getModernDeemphColorMapFunc( mdcmxs );
+
 		xr *= mult;
 		yr *= mult;
 		xscale=yscale=1;
@@ -528,7 +570,7 @@ void Blit8ToHigh(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale,
 			{
 				for(x=xr;x;x--)
 				{
-				 *(uint32 *)dest=ModernDeemphColorMap(src,specbuf8bpp,mdcmxs, mdcmys);
+				 *(uint32 *)dest=ModernDeemphColorMapFunc(src,specbuf8bpp);
 				 dest+=4;
 				 src++;
 				}
@@ -541,7 +583,7 @@ void Blit8ToHigh(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale,
 			{
 				for(x=xr;x;x--)
 				{
-					uint32 tmp=ModernDeemphColorMap(src,specbuf8bpp,mdcmxs, mdcmys);
+					uint32 tmp=ModernDeemphColorMapFunc(src,specbuf8bpp);
 					*(uint8 *)dest=tmp;
 					*((uint8 *)dest+1)=tmp>>8;
 					*((uint8 *)dest+2)=tmp>>16;
@@ -581,7 +623,7 @@ void Blit8ToHigh(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale,
 		{
 			for(x=xr; x; x--)
 			{
-				*(uint32 *)dest = ModernDeemphColorMap(src,XBuf,1,1);
+				*(uint32 *)dest = _ModernDeemphColorMap<1>(src,XBuf);
 				dest += 4;
 				src++;
 			}
@@ -592,9 +634,11 @@ void Blit8ToHigh(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale,
 		{
 			uint32 *s = prescalebuf;
 			uint32 *d = (uint32 *)destbackup; // use 32-bit pointers ftw
-			int subpixel;
+			int subpixel,yend;
 
-			for (y=0; y<yr*yscale; y++)
+			yend = yr*yscale;
+
+			for (y=0; y<yend; y++)
 			{
 				int back = xr*(y%yscale>0); // bool as multiplier
 				for (x=0; x<xr; x++)
@@ -790,6 +834,7 @@ void Blit8ToHigh(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale,
 						lastindex = index;
 					}
 				}
+				src += (256-xr);
 			}
 
 		}
@@ -919,7 +964,7 @@ void Blit8ToHigh(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale,
 					for(x=xr;x;x--)
 					{
 						//THE MAIN BLITTING CODEPATH (there may be others that are important)
-						*(uint32 *)dest = ModernDeemphColorMap(src,XBuf,1,1);
+						*(uint32 *)dest = _ModernDeemphColorMap<1>(src,XBuf);
 						dest+=4;
 						src++;
 					}
@@ -932,7 +977,7 @@ void Blit8ToHigh(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale,
 				{
 					for(x=xr;x;x--)
 					{     
-						uint32 tmp = ModernDeemphColorMap(src,XBuf,1,1);
+						uint32 tmp = _ModernDeemphColorMap<1>(src,XBuf);
 						*(uint8 *)dest=tmp;
 						*((uint8 *)dest+1)=tmp>>8;
 						*((uint8 *)dest+2)=tmp>>16;
@@ -948,7 +993,7 @@ void Blit8ToHigh(uint8 *src, uint8 *dest, int xr, int yr, int pitch, int xscale,
 				{
 					for(x=xr;x;x--)
 					{
-						*(uint16 *)dest = ModernDeemphColorMap(src,XBuf,1,1);
+						*(uint16 *)dest = _ModernDeemphColorMap<1>(src,XBuf);
 						dest+=2;
 						src++;
 					}
