@@ -43,7 +43,7 @@
 #include "file.h"
 #include "vsuni.h"
 #include "ines.h"
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 #include "drivers/win/pref.h"
 #include "utils/xstring.h"
 
@@ -65,7 +65,7 @@ extern void RefreshThrottleFPS();
 #endif
 
 //TODO - we really need some kind of global platform-specific options api
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 #include "drivers/win/main.h"
 #include "drivers/win/memview.h"
 #include "drivers/win/cheat.h"
@@ -120,19 +120,23 @@ char romNameWhenClosingEmulator[2048] = {0};
 
 FCEUGI::FCEUGI()
 	: filename(0),
-	  archiveFilename(0) {
+	  archiveFilename(0) 
+{
 	//printf("%08x",opsize); // WTF?!
 }
 
-FCEUGI::~FCEUGI() {
-	if (filename) {
-        free(filename);
-        filename = NULL;
-    }
-	if (archiveFilename) {
-        delete archiveFilename;
-        archiveFilename = NULL;
-    }
+FCEUGI::~FCEUGI() 
+{
+	if (filename) 
+	{
+		free(filename);
+		filename = NULL;
+	}
+	if (archiveFilename) 
+	{
+		free(archiveFilename);
+		archiveFilename = NULL;
+	}
 }
 
 bool CheckFileExists(const char* filename) {
@@ -161,7 +165,7 @@ void FCEU_TogglePPU(void) {
 		FCEUI_printf("Old PPU loaded");
 	}
 	normalscanlines = (dendy ? 290 : 240)+newppu; // use flag as number!
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	SetMainWindowText();
 #endif
 }
@@ -176,8 +180,7 @@ static void FCEU_CloseGame(void)
 			FCEUSS_Save(FCEU_MakeFName(FCEUMKF_RESUMESTATE, 0, 0).c_str(), false);
 		}
 
-#ifdef WIN32
-		extern char LoadedRomFName[2048];
+#ifdef __WIN_DRIVER__
 		if (storePreferences(mass_replace(LoadedRomFName, "|", ".").c_str()))
 			FCEUD_PrintError("Couldn't store debugging data");
 		CDLoggerROMClosed();
@@ -193,7 +196,7 @@ static void FCEU_CloseGame(void)
 		}
 
 		if (GameInfo->type != GIT_NSF) {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 			if (disableAutoLSCheats == 2)
 				FCEU_FlushGameCheats(0, 1);
 			else if (disableAutoLSCheats == 1)
@@ -222,10 +225,10 @@ static void FCEU_CloseGame(void)
 		currFrameCounter = 0;
 
 		//Reset flags for Undo/Redo/Auto Savestating //adelikat: TODO: maybe this stuff would be cleaner as a struct or class
-		lastSavestateMade[0] = 0;
+		lastSavestateMade.clear();
 		undoSS = false;
 		redoSS = false;
-		lastLoadstateMade[0] = 0;
+		lastLoadstateMade.clear();
 		undoLS = false;
 		redoLS = false;
 		AutoSS = false;
@@ -417,17 +420,17 @@ FCEUGI *FCEUI_LoadGameVirtual(const char *name, int OverwriteVidMode, bool silen
 	//----------
 	//attempt to open the files
 	FCEUFILE *fp;
-	char fullname[2048];	// this name contains both archive name and ROM file name
+	std::string fullname;	// this name contains both archive name and ROM file name
 	int lastpal = PAL;
 	int lastdendy = dendy;
 
-	const char* romextensions[] = { "nes", "fds", 0 };
+	const char* romextensions[] = { "nes", "fds", "nsf", 0 };
 
 	// indicator for if the operaton was canceled by user
 	// currently there's only one situation:
 	// the user clicked cancel form the open from archive dialog
 	int userCancel = 0;
-	fp = FCEU_fopen(name, 0, "rb", 0, -1, romextensions, &userCancel);
+	fp = FCEU_fopen(name, LoadedRomFNamePatchToUse[0] ? LoadedRomFNamePatchToUse : nullptr, "rb", 0, -1, romextensions, &userCancel);
 
 	if (!fp)
 	{
@@ -439,16 +442,19 @@ FCEUGI *FCEUI_LoadGameVirtual(const char *name, int OverwriteVidMode, bool silen
 	}
 	else if (fp->archiveFilename != "")
 	{
-		strcpy(fullname, fp->archiveFilename.c_str());
-		strcat(fullname, "|");
-		strcat(fullname, fp->filename.c_str());
-	} else
-		strcpy(fullname, name);
+		fullname.assign(fp->archiveFilename.c_str());
+		fullname.append("|");
+		fullname.append(fp->filename.c_str());
+	}
+	else
+	{
+		fullname.assign(name);
+	}
 
 	// reset loaded game BEFORE it's loading.
 	ResetGameLoaded();
 	//file opened ok. start loading.
-	FCEU_printf("Loading %s...\n\n", fullname);
+	FCEU_printf("Loading %s...\n\n", fullname.c_str());
 	GetFileBase(fp->filename.c_str());
 	//reset parameters so they're cleared just in case a format's loader doesn't know to do the clearing
 	MasterRomInfoParams = TMasterRomInfoParams();
@@ -460,7 +466,7 @@ FCEUGI *FCEUI_LoadGameVirtual(const char *name, int OverwriteVidMode, bool silen
 
 	FCEU_CloseGame();
 	GameInfo = new FCEUGI();
-	memset(GameInfo, 0, sizeof(FCEUGI));
+	memset( (void*)GameInfo, 0, sizeof(FCEUGI));
 
 	GameInfo->filename = strdup(fp->filename.c_str());
 	if (fp->archiveFilename != "")
@@ -480,25 +486,24 @@ FCEUGI *FCEUI_LoadGameVirtual(const char *name, int OverwriteVidMode, bool silen
 	bool FCEUXLoad(const char *name, FCEUFILE * fp);
 
 	int load_result;
-	load_result = iNESLoad(fullname, fp, OverwriteVidMode);
+	load_result = iNESLoad(fullname.c_str(), fp, OverwriteVidMode);
 	if (load_result == LOADER_INVALID_FORMAT)
 	{
-		load_result = NSFLoad(fullname, fp);
+		load_result = NSFLoad(fullname.c_str(), fp);
 		if (load_result == LOADER_INVALID_FORMAT)
 		{
-			load_result = UNIFLoad(fullname, fp);
+			load_result = UNIFLoad(fullname.c_str(), fp);
 			if (load_result == LOADER_INVALID_FORMAT)
 			{
-				load_result = FDSLoad(fullname, fp);
+				load_result = FDSLoad(fullname.c_str(), fp);
 			}
 		}
 	}	
 	if (load_result == LOADER_OK)
 	{
 
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 		// ################################## Start of SP CODE ###########################
-		extern char LoadedRomFName[2048];
 		extern int loadDebugDataFailed;
 
 		if ((loadDebugDataFailed = loadPreferences(mass_replace(LoadedRomFName, "|", ".").c_str())))
@@ -516,7 +521,7 @@ FCEUGI *FCEUI_LoadGameVirtual(const char *name, int OverwriteVidMode, bool silen
 			FCEU_OpenGenie())
 		{
 			FCEUI_SetGameGenie(false);
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 			genie = 0;
 #endif
 		}
@@ -531,16 +536,16 @@ FCEUGI *FCEUI_LoadGameVirtual(const char *name, int OverwriteVidMode, bool silen
 
 		if (!lastpal && PAL) {
 			FCEU_DispMessage("PAL mode set", 0);
-			FCEUI_printf("PAL mode set");
+			FCEUI_printf("PAL mode set\n");
 		}
 		else if (!lastdendy && dendy) {
 			// this won't happen, since we don't autodetect dendy, but maybe someday we will?
 			FCEU_DispMessage("Dendy mode set", 0);
-			FCEUI_printf("Dendy mode set");
+			FCEUI_printf("Dendy mode set\n");
 		}
 		else if ((lastpal || lastdendy) && !(PAL || dendy)) {
 			FCEU_DispMessage("NTSC mode set", 0);
-			FCEUI_printf("NTSC mode set");
+			FCEUI_printf("NTSC mode set\n");
 		}
 
 		if (GameInfo->type != GIT_NSF && !disableAutoLSCheats)
@@ -555,7 +560,7 @@ FCEUGI *FCEUI_LoadGameVirtual(const char *name, int OverwriteVidMode, bool silen
 
 		ResetScreenshotsCounter();
 
-#if defined (WIN32) || defined (WIN64)
+#ifdef __WIN_DRIVER__
 		DoDebuggerDataReload(); // Reloads data without reopening window
 		CDLoggerROMChanged();
 		if (hMemView) UpdateColorTable();
@@ -640,62 +645,115 @@ void FCEUI_Kill(void) {
 }
 
 int rapidAlternator = 0;
-int AutoFirePattern[8] = { 1, 0, 0, 0, 0, 0, 0, 0 };
+//int AutoFirePattern[8] = { 1, 0, 0, 0, 0, 0, 0, 0 };
 int AutoFirePatternLength = 2;
 
-void SetAutoFirePattern(int onframes, int offframes) {
-	int i;
-	for (i = 0; i < onframes && i < 8; i++) {
-		AutoFirePattern[i] = 1;
-	}
-	for (; i < 8; i++) {
-		AutoFirePattern[i] = 0;
-	}
-	if (onframes + offframes < 2) {
-		AutoFirePatternLength = 2;
-	} else if (onframes + offframes > 8) {
-		AutoFirePatternLength = 8;
-	} else {
-		AutoFirePatternLength = onframes + offframes;
-	}
+void SetAutoFirePattern(int onframes, int offframes) 
+{
+	//int i;
+	//for (i = 0; i < onframes && i < 8; i++) {
+	//	AutoFirePattern[i] = 1;
+	//}
+	//for (; i < 8; i++) {
+	//	AutoFirePattern[i] = 0;
+	//}
+	//if (onframes + offframes < 2) {
+	//	AutoFirePatternLength = 2;
+	//} else if (onframes + offframes > 8) {
+	//	AutoFirePatternLength = 8;
+	//} else {
+	//	AutoFirePatternLength = onframes + offframes;
+	//}
+	AutoFirePatternLength = onframes + offframes;
 	AFon = onframes; AFoff = offframes;
 }
 
-void SetAutoFireOffset(int offset) {
+void GetAutoFirePattern( int *onframes, int *offframes)
+{
+	if ( onframes )
+	{
+		*onframes = AFon;
+	}
+	if ( offframes )
+	{
+		*offframes = AFoff;
+	}
+}
+
+void SetAutoFireOffset(int offset)
+{
 	if (offset < 0 || offset > 8) return;
 	AutoFireOffset = offset;
 }
 
-void AutoFire(void) {
+bool GetAutoFireState(int btnIdx)
+{
+	return rapidAlternator;
+}
+
+void AutoFire(void) 
+{
 	static int counter = 0;
 	if (justLagged == false)
-		counter = (counter + 1) % (8 * 7 * 5 * 3);
+	{
+		//counter = (counter + 1) % (8 * 7 * 5 * 3);
+		counter = (counter + 1) % AutoFirePatternLength;
+	}
 	//If recording a movie, use the frame # for the autofire so the offset
 	//doesn't get screwed up when loading.
-	if (FCEUMOV_Mode(MOVIEMODE_RECORD | MOVIEMODE_PLAY)) {
-		rapidAlternator = AutoFirePattern[(AutoFireOffset + FCEUMOV_GetFrame()) % AutoFirePatternLength]; //adelikat: TODO: Think through this, MOVIEMODE_FINISHED should not use movie data for auto-fire?
-	} else {
-		rapidAlternator = AutoFirePattern[(AutoFireOffset + counter) % AutoFirePatternLength];
+	if (FCEUMOV_Mode(MOVIEMODE_RECORD | MOVIEMODE_PLAY)) 
+	{
+		//rapidAlternator = AutoFirePattern[(AutoFireOffset + FCEUMOV_GetFrame()) % AutoFirePatternLength]; //adelikat: TODO: Think through this, MOVIEMODE_FINISHED should not use movie data for auto-fire?
+		//adelikat: TODO: Think through this, MOVIEMODE_FINISHED should not use movie data for auto-fire?
+		rapidAlternator = ( (AutoFireOffset + FCEUMOV_GetFrame()) % AutoFirePatternLength ) < AFon; 
+	}
+	else
+	{
+		//rapidAlternator = AutoFirePattern[(AutoFireOffset + counter) % AutoFirePatternLength];
+		rapidAlternator = ( (AutoFireOffset + counter) % AutoFirePatternLength ) < AFon;
 	}
 }
 
 void UpdateAutosave(void);
 
+
+#ifdef __QT_DRIVER__
+extern unsigned int frameAdvHoldTimer;
+#endif
+
 ///Emulates a single frame.
 
 ///Skip may be passed in, if FRAMESKIP is #defined, to cause this to emulate more than one frame
 void FCEUI_Emulate(uint8 **pXBuf, int32 **SoundBuf, int32 *SoundBufSize, int skip) {
 	//skip initiates frame skip if 1, or frame skip and sound skip if 2
-	int r, ssize;
+	FCEU_MAYBE_UNUSED int r;
+	int ssize;
 
 	JustFrameAdvanced = false;
 
 	if (frameAdvanceRequested)
 	{
+#ifdef __QT_DRIVER__
+		uint32_t frameAdvanceDelayScaled = frameAdvance_Delay * (PAL ? 20 : 16);
+
+		if ( frameAdvanceDelayScaled < 1 )
+		{
+			frameAdvanceDelayScaled = 1;
+		}
+		if ( (frameAdvance_Delay_count == 0) || (frameAdvHoldTimer >= frameAdvanceDelayScaled) )
+		{
+			EmulationPaused = EMULATIONPAUSED_FA;
+		}
+		if ( static_cast<unsigned int>(frameAdvance_Delay_count) < frameAdvanceDelayScaled)
+		{
+			frameAdvance_Delay_count++;
+		}
+#else
 		if (frameAdvance_Delay_count == 0 || frameAdvance_Delay_count >= frameAdvance_Delay)
 			EmulationPaused = EMULATIONPAUSED_FA;
 		if (frameAdvance_Delay_count < frameAdvance_Delay)
 			frameAdvance_Delay_count++;
+#endif
 	}
 
 	if (EmulationPaused & EMULATIONPAUSED_FA)
@@ -703,7 +761,7 @@ void FCEUI_Emulate(uint8 **pXBuf, int32 **SoundBuf, int32 *SoundBufSize, int ski
 		// the user is holding Frame Advance key
 		// clear paused flag temporarily
 		EmulationPaused &= ~EMULATIONPAUSED_PAUSED;
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 		// different emulation speed when holding Frame Advance
 		if (fps_scale_frameadvance > 0)
 		{
@@ -713,7 +771,7 @@ void FCEUI_Emulate(uint8 **pXBuf, int32 **SoundBuf, int32 *SoundBufSize, int ski
 #endif
 	} else
 	{
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 		if (fps_scale_frameadvance > 0)
 		{
 			// restore emulation speed when Frame Advance is not held
@@ -752,13 +810,16 @@ void FCEUI_Emulate(uint8 **pXBuf, int32 **SoundBuf, int32 *SoundBufSize, int ski
 
 	if (skip != 2) ssize = FlushEmulateSound();  //If skip = 2 we are skipping sound processing
 
+	//flush tracer once a frame, since we're likely to end up back at a user interaction loop after this with emulation paused
+	FCEUD_FlushTrace();
+
 #ifdef _S9XLUA_H
 	CallRegisteredLuaFunctions(LUACALL_AFTEREMULATION);
 #endif
 
 	FCEU_PutImage();
 
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	//These Windows only dialogs need to be updated only once per frame so they are included here
 	// CaH4e3: can't see why, this is only cause problems with selection
 	// adelikat: selection is only a problem when not paused, it should be paused to select, we want to see the values update
@@ -797,7 +858,7 @@ void FCEUI_Emulate(uint8 **pXBuf, int32 **SoundBuf, int32 *SoundBufSize, int ski
 	{
 		EmulationPaused = EMULATIONPAUSED_PAUSED;		   // restore EMULATIONPAUSED_PAUSED flag and clear EMULATIONPAUSED_FA flag
 		JustFrameAdvanced = true;
-		#ifdef WIN32
+		#ifdef __WIN_DRIVER__
 		if (soundoptions & SO_MUTEFA)  //mute the frame advance if the user requested it
 			*SoundBufSize = 0;         //keep sound muted
 		#endif
@@ -962,7 +1023,7 @@ void PowerNES(void) {
 
 	timestampbase = 0;
 	X6502_Power();
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	ResetDebugStatisticsCounters();
 #endif
 	FCEU_PowerCheats();
@@ -971,7 +1032,7 @@ void PowerNES(void) {
 	extern uint8 *XBackBuf;
 	memset(XBackBuf, 0, 256 * 256);
 
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	Update_RAM_Search(); // Update_RAM_Watch() is also called.
 #endif
 
@@ -1005,7 +1066,7 @@ void FCEU_ResetVidSys(void) {
 
 FCEUS FSettings;
 
-void FCEU_printf(const char *format, ...) 
+void FCEU_printf( __FCEU_PRINTF_FORMAT const char *format, ...)
 {
 	char temp[2048];
 
@@ -1025,7 +1086,7 @@ void FCEU_printf(const char *format, ...)
 	va_end(ap);
 }
 
-void FCEU_PrintError(const char *format, ...) 
+void FCEU_PrintError( __FCEU_PRINTF_FORMAT const char *format, ...)
 {
 	char temp[2048];
 
@@ -1069,7 +1130,27 @@ int FCEUI_GetCurrentVidSystem(int *slstart, int *slend) {
 	return(PAL);
 }
 
-void FCEUI_SetRegion(int region, int notify) {
+int  FCEUI_GetRegion(void)
+{
+	int region;
+
+	if ( pal_emulation )
+	{
+		region = 1;
+	}
+	else if ( dendy )
+	{
+		region = 2;
+	}
+	else
+	{
+		region = 0;
+	}
+	return region;
+}
+
+void FCEUI_SetRegion(int region, int notify) 
+{
 	switch (region) {
 		case 0: // NTSC
 			normalscanlines = 240;
@@ -1079,7 +1160,7 @@ void FCEUI_SetRegion(int region, int notify) {
 			if (notify)
 			{
 				FCEU_DispMessage("NTSC mode set", 0);
-				FCEUI_printf("NTSC mode set");
+				FCEUI_printf("NTSC mode set\n");
 			}
 			break;
 		case 1: // PAL
@@ -1090,7 +1171,7 @@ void FCEUI_SetRegion(int region, int notify) {
 			if (notify)
 			{
 				FCEU_DispMessage("PAL mode set", 0);
-				FCEUI_printf("PAL mode set");
+				FCEUI_printf("PAL mode set\n");
 			}
 			break;
 		case 2: // Dendy
@@ -1101,7 +1182,7 @@ void FCEUI_SetRegion(int region, int notify) {
 			if (notify)
 			{
 				FCEU_DispMessage("Dendy mode set", 0);
-				FCEUI_printf("Dendy mode set");
+				FCEUI_printf("Dendy mode set\n");
 			}
 			break;
 	}
@@ -1109,7 +1190,7 @@ void FCEUI_SetRegion(int region, int notify) {
 	totalscanlines = normalscanlines + (overclock_enabled ? postrenderscanlines : 0);
 	FCEUI_SetVidSystem(pal_emulation);
 	RefreshThrottleFPS();
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	UpdateCheckedMenuItems();
 	PushCurrentVideoSettings();
 #endif
@@ -1152,12 +1233,16 @@ void FCEUI_ClearEmulationFrameStepped()
 //ideally maybe we shouldnt be using this, but i need it for quick merging
 void FCEUI_SetEmulationPaused(int val) {
 	EmulationPaused = val;
+	if(EmulationPaused)
+		FCEUD_FlushTrace();
 }
 
 void FCEUI_ToggleEmulationPause(void)
 {
 	EmulationPaused = (EmulationPaused & EMULATIONPAUSED_PAUSED) ^ EMULATIONPAUSED_PAUSED;
 	DebuggerWasUpdated = false;
+	if(EmulationPaused)
+		FCEUD_FlushTrace();
 }
 
 void FCEUI_FrameAdvanceEnd(void) {
@@ -1165,8 +1250,8 @@ void FCEUI_FrameAdvanceEnd(void) {
 }
 
 void FCEUI_FrameAdvance(void) {
-	frameAdvanceRequested = true;
 	frameAdvance_Delay_count = 0;
+	frameAdvanceRequested = true;
 }
 
 static int AutosaveCounter = 0;
@@ -1183,7 +1268,7 @@ void UpdateAutosave(void) {
 		FCEUSS_Save(f, false);
 		AutoSS = true;  //Flag that an auto-savestate was made
 		free(f);
-        f = NULL;
+		f = NULL;
 		AutosaveStatus[AutosaveIndex] = 1;
 	}
 }
@@ -1260,7 +1345,7 @@ bool FCEU_IsValidUI(EFCEUI ui) {
 	case FCEUI_INSERT_COIN:
 		if (!GameInfo) return false;
 		if (FCEUMOV_Mode(MOVIEMODE_RECORD)) return true;
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 		if (FCEUMOV_Mode(MOVIEMODE_TASEDITOR) && isTaseditorRecording()) return true;
 #endif
 		if (!FCEUMOV_Mode(MOVIEMODE_INACTIVE)) return false;
@@ -1269,6 +1354,10 @@ bool FCEU_IsValidUI(EFCEUI ui) {
 	case FCEUI_INPUT_BARCODE:
 		if (!GameInfo) return false;
 		if (!FCEUMOV_Mode(MOVIEMODE_INACTIVE)) return false;
+		break;
+	default:
+		// Unhandled falls out to end of function
+		break;
 	}
 
 	return true;
@@ -1330,10 +1419,16 @@ virtual void Power() {
 }
 };
 
-void FCEUXGameInterface(GI command) {
-	switch (command) {
-	case GI_POWER:
-		cart->Power();
+void FCEUXGameInterface(GI command)
+{
+	switch (command)
+	{
+		case GI_POWER:
+			cart->Power();
+		break;
+		default:
+			// Unhandled cases
+		break;
 	}
 }
 
@@ -1401,7 +1496,7 @@ uint8 FCEU_ReadRomByte(uint32 i) {
 
 void FCEU_WriteRomByte(uint32 i, uint8 value) {
 	if (i < 16)
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 		MessageBox(hMemView, "Sorry", "You can't edit the ROM header.", MB_OK | MB_ICONERROR);
 #else
 		printf("Sorry, you can't edit the ROM header.\n");
