@@ -69,7 +69,7 @@ void ApplyIPS(FILE *ips, FCEUFILE* fp)
 
 	if(!ips) return;
 
-	char* buf = (char*)FCEU_dmalloc(fp->size);
+	char* buf = (char*)FCEU_malloc(fp->size);
 	memcpy(buf,fp->EnsureMemorystream()->buf(),fp->size);
 
 
@@ -108,13 +108,7 @@ void ApplyIPS(FILE *ips, FCEUFILE* fp)
 			if((offset+size)>(uint32)fp->size)
 			{
 				// Probably a little slow.
-				char *newbuf=(char *)realloc(buf,offset+size);
-				if(!newbuf)
-				{
-					free(buf); buf=NULL;
-					FCEU_printf("  Oops.  IPS patch %d(type RLE) goes beyond end of file.  Could not allocate memory.\n",count);
-					goto end;
-				}
+				char *newbuf=(char *)FCEU_realloc(buf,offset+size);
 				buf=newbuf;
 				memset(buf+fp->size,0,offset+size-fp->size);
 				fp->size=offset+size;
@@ -133,17 +127,15 @@ void ApplyIPS(FILE *ips, FCEUFILE* fp)
 			if((offset+size)>(uint32)fp->size)
 			{
 				// Probably a little slow.
-				char *newbuf=(char *)realloc(buf,offset+size);
-				if(!newbuf)
-				{
-					free(buf); buf=NULL;
-					FCEU_printf("  Oops.  IPS patch %d(type normal) goes beyond end of file.  Could not allocate memory.\n",count);
-					goto end;
-				}
+				char *newbuf=(char *)FCEU_realloc(buf,offset+size);
 				buf=newbuf;
 				memset(buf+fp->size,0,offset+size-fp->size);
+				fp->size=offset+size;
+			}
+			if ( fread(buf+offset,1,size,ips) != static_cast<size_t>(size) )
+			{
+				FCEU_printf(" Warn IPS data read came up short!\n");
 			}
-			fread(buf+offset,1,size,ips);
 		}
 		count++;
 	}
@@ -287,6 +279,13 @@ FCEUFILE * FCEU_fopen(const char *path, const char *ipsfn, const char *mode, cha
 	if(read)
 	{
 		ArchiveScanRecord asr = FCEUD_ScanArchive(fileToOpen);
+		if (asr.numFilesInArchive < 0)
+		{
+			// error occurred, return
+			// actually it's canceled not by user but an error message already shown
+			*userCancel = 1;
+			return fceufp;
+		}
 		asr.files.FilterByExtension(extensions);
 		if(!asr.isArchive())
 		{
@@ -319,9 +318,9 @@ FCEUFILE * FCEU_fopen(const char *path, const char *ipsfn, const char *mode, cha
 			{
 				uint32 magic;
 
-				magic = fp->fgetc();
-				magic|=fp->fgetc()<<8;
-				magic|=fp->fgetc()<<16;
+				magic = (fp->fgetc() & 0x00ff);
+				magic|= (fp->fgetc() & 0x00ff) << 8;
+				magic|= (fp->fgetc() & 0x00ff) << 16;
 				fp->fseek(0,SEEK_SET);
 
 				if(magic==0x088b1f) {
@@ -331,7 +330,7 @@ FCEUFILE * FCEU_fopen(const char *path, const char *ipsfn, const char *mode, cha
 					if(gzfile) {
 						delete fp;
 
-						int size;
+						size_t size;
 						for(size=0; gzgetc(gzfile) != EOF; size++) {}
 						EMUFILE_MEMORY* ms = new EMUFILE_MEMORY(size);
 						gzseek(gzfile,0,SEEK_SET);
@@ -450,13 +449,12 @@ int FCEU_fisarchive(FCEUFILE *fp)
 std::string GetMfn() //Retrieves the movie filename from curMovieFilename (for adding to savestate and auto-save files)
 {
 	std::string movieFilenamePart;
-	extern char curMovieFilename[512];
-	if(*curMovieFilename)
-		{
+	if (!curMovieFilename.empty())
+	{
 		char drv[PATH_MAX], dir[PATH_MAX], name[PATH_MAX], ext[PATH_MAX];
-		splitpath(curMovieFilename,drv,dir,name,ext);
+		splitpath(curMovieFilename.c_str(),drv,dir,name,ext);
 		movieFilenamePart = std::string(".") + name;
-		}
+	}
 	return movieFilenamePart;
 }
 
@@ -488,7 +486,7 @@ void FCEUI_SetDirOverride(int which, char *n)
 		va_list ap;
 		int ret;
 
-		if(!(*strp=(char*)FCEU_dmalloc(2048))) //mbg merge 7/17/06 cast to char*
+		if(!(*strp=(char*)FCEU_malloc(2048))) //mbg merge 7/17/06 cast to char*
 			return(0);
 		va_start(ap,fmt);
 		ret=vsnprintf(*strp,2048,fmt,ap);
