@@ -106,6 +106,8 @@ uint8 FCEU_GetJoyJoy(void)
 }
 
 extern uint8 coinon;
+extern uint8 coinon2;
+extern uint8 service;
 
 //set to true if the fourscore is attached
 static bool FSAttached = false;
@@ -431,8 +433,11 @@ void FCEU_UpdateInput(void)
 		portFC.driver->Update(portFC.ptr,portFC.attrib);
 	}
 
-	if(GameInfo->type==GIT_VSUNI)
-		if(coinon) coinon--;
+	if (GameInfo->type == GIT_VSUNI) {
+		if (coinon) coinon--;
+		if (coinon2) coinon2--;
+		if (service) service--;
+	}
 
 	if(FCEUnetplay)
 		NetplayUpdate(joy);
@@ -454,7 +459,11 @@ static DECLFR(VSUNIRead0)
 
 	ret|=(vsdip&3)<<3;
 	if(coinon)
-		ret|=0x4;
+		ret |= 0x20;
+	if (coinon2)
+		ret |= 0x40;
+	if (service)
+		ret |= 0x04;
 	return ret;
 }
 
@@ -486,9 +495,16 @@ static void SetInputStuff(int port)
 	switch(joyports[port].type)
 	{
 	case SI_GAMEPAD:
-		if(GameInfo->type==GIT_VSUNI){
-			joyports[port].driver = &GPCVS;
-		} else {
+		if (GameInfo)
+		{
+			if (GameInfo->type==GIT_VSUNI){
+				joyports[port].driver = &GPCVS;
+			} else {
+				joyports[port].driver= &GPC;
+			}
+		}
+		else
+		{
 			joyports[port].driver= &GPC;
 		}
 		break;
@@ -661,7 +677,9 @@ void FCEU_DoSimpleCommand(int cmd)
 	{
 	case FCEUNPCMD_FDSINSERT: FCEU_FDSInsert();break;
 	case FCEUNPCMD_FDSSELECT: FCEU_FDSSelect();break;
-	case FCEUNPCMD_VSUNICOIN: FCEU_VSUniCoin(); break;
+	case FCEUNPCMD_VSUNICOIN: FCEU_VSUniCoin(0); break;
+	case FCEUNPCMD_VSUNICOIN2: FCEU_VSUniCoin(1); break;
+	case FCEUNPCMD_VSUNISERVICE: FCEU_VSUniService(); break;
 	case FCEUNPCMD_VSUNIDIP0:
 	case FCEUNPCMD_VSUNIDIP0+1:
 	case FCEUNPCMD_VSUNIDIP0+2:
@@ -719,6 +737,22 @@ void FCEUI_VSUniCoin(void)
 	FCEU_QSimpleCommand(FCEUNPCMD_VSUNICOIN);
 }
 
+void FCEUI_VSUniCoin2(void)
+{
+	if (!FCEU_IsValidUI(FCEUI_INSERT_COIN))
+		return;
+
+	FCEU_QSimpleCommand(FCEUNPCMD_VSUNICOIN2);
+}
+
+void FCEUI_VSUniService(void)
+{
+	if (!FCEU_IsValidUI(FCEUI_INSERT_COIN))
+		return;
+
+	FCEU_QSimpleCommand(FCEUNPCMD_VSUNISERVICE);
+}
+
 //Resets the frame counter if movie inactive and rom is reset or power-cycle
 void ResetFrameCounter()
 {
@@ -791,6 +825,7 @@ static void RamSearchOpLTE(void);
 static void RamSearchOpGTE(void);
 static void RamSearchOpEQ(void);
 static void RamSearchOpNE(void);
+static void ToggleCheats(void);
 static void DebuggerStepInto(void);
 static void FA_SkipLag(void);
 static void OpenRom(void);
@@ -893,17 +928,17 @@ struct EMUCMDTABLE FCEUI_CommandTable[]=
 	{ EMUCMD_FDS_EJECT_INSERT,				EMUCMDTYPE_FDS,		FCEUI_FDSInsert,				0, 0, "Eject or Insert FDS Disk", EMUCMDFLAG_TASEDITOR },
 	{ EMUCMD_FDS_SIDE_SELECT,				EMUCMDTYPE_FDS,		FCEUI_FDSSelect,				0, 0, "Switch FDS Disk Side", EMUCMDFLAG_TASEDITOR },
 
-	{ EMUCMD_VSUNI_COIN,					EMUCMDTYPE_VSUNI,	FCEUI_VSUniCoin,				0, 0, "Insert Coin", EMUCMDFLAG_TASEDITOR },
-	{ EMUCMD_VSUNI_TOGGLE_DIP_0,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dipswitch 0", 0 },
-	{ EMUCMD_VSUNI_TOGGLE_DIP_1,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dipswitch 1", 0 },
-	{ EMUCMD_VSUNI_TOGGLE_DIP_2,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dipswitch 2", 0 },
-	{ EMUCMD_VSUNI_TOGGLE_DIP_3,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dipswitch 3", 0 },
-	{ EMUCMD_VSUNI_TOGGLE_DIP_4,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dipswitch 4", 0 },
-	{ EMUCMD_VSUNI_TOGGLE_DIP_5,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dipswitch 5", 0 },
-	{ EMUCMD_VSUNI_TOGGLE_DIP_6,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dipswitch 6", 0 },
-	{ EMUCMD_VSUNI_TOGGLE_DIP_7,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dipswitch 7", 0 },
-	{ EMUCMD_VSUNI_TOGGLE_DIP_8,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dipswitch 8", 0 },
-	{ EMUCMD_VSUNI_TOGGLE_DIP_9,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dipswitch 9", 0 },
+	{ EMUCMD_VSUNI_COIN,					EMUCMDTYPE_VSUNI,	FCEUI_VSUniCoin,				0, 0, "Insert Coin #1", EMUCMDFLAG_TASEDITOR },
+	{ EMUCMD_VSUNI_COIN_2,					EMUCMDTYPE_VSUNI,	FCEUI_VSUniCoin2,				0, 0, "Insert Coin #2", EMUCMDFLAG_TASEDITOR },
+	{ EMUCMD_VSUNI_SERVICE_BUTTON,			EMUCMDTYPE_VSUNI,	FCEUI_VSUniService,				0, 0, "Service Button", EMUCMDFLAG_TASEDITOR },
+	{ EMUCMD_VSUNI_TOGGLE_DIP_0,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dip Switch 0", 0 },
+	{ EMUCMD_VSUNI_TOGGLE_DIP_1,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dip Switch 1", 0 },
+	{ EMUCMD_VSUNI_TOGGLE_DIP_2,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dip Switch 2", 0 },
+	{ EMUCMD_VSUNI_TOGGLE_DIP_3,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dip Switch 3", 0 },
+	{ EMUCMD_VSUNI_TOGGLE_DIP_4,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dip Switch 4", 0 },
+	{ EMUCMD_VSUNI_TOGGLE_DIP_5,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dip Switch 5", 0 },
+	{ EMUCMD_VSUNI_TOGGLE_DIP_6,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dip Switch 6", 0 },
+	{ EMUCMD_VSUNI_TOGGLE_DIP_7,			EMUCMDTYPE_VSUNI,	CommandToggleDip,				0, 0, "Toggle Dip Switch 7", 0 },
 
 	{ EMUCMD_MISC_AUTOSAVE,					EMUCMDTYPE_MISC,	FCEUI_RewindToLastAutosave,		0, 0, "Load Last Auto-save", 0},
 	{ EMUCMD_MISC_SHOWSTATES,				EMUCMDTYPE_MISC,	ViewSlots,						0, 0, "View save slots", 0 },
@@ -938,6 +973,7 @@ struct EMUCMDTABLE FCEUI_CommandTable[]=
 	{ EMUCMD_TOOL_RAMSEARCHGTE,				EMUCMDTYPE_TOOL,	RamSearchOpGTE,					0, 0, "Ram Search - Greater Than or Equal", 0},
 	{ EMUCMD_TOOL_RAMSEARCHEQ,				EMUCMDTYPE_TOOL,	RamSearchOpEQ,					0, 0, "Ram Search - Equal",	  0},
 	{ EMUCMD_TOOL_RAMSEARCHNE,				EMUCMDTYPE_TOOL,	RamSearchOpNE,					0, 0, "Ram Search - Not Equal", 0},
+	{ EMUCMD_TOOL_TOGGLECHEATS,				EMUCMDTYPE_TOOL,	ToggleCheats,					0, 0, "Toggle Cheats", 0},
 	{ EMUCMD_RERECORD_DISPLAY_TOGGLE,		EMUCMDTYPE_MISC,   FCEUI_MovieToggleRerecordDisplay,0, 0, "Toggle Rerecord Display", EMUCMDFLAG_TASEDITOR },
 
 	{ EMUCMD_TASEDITOR_REWIND,				EMUCMDTYPE_TASEDITOR,	TaseditorRewindOn,			TaseditorRewindOff, 0, "Frame Rewind", EMUCMDFLAG_TASEDITOR },
@@ -953,12 +989,12 @@ struct EMUCMDTABLE FCEUI_CommandTable[]=
 
 #define NUM_EMU_CMDS		(sizeof(FCEUI_CommandTable)/sizeof(FCEUI_CommandTable[0]))
 
-static int execcmd, i;
+static int execcmd;
 
 void FCEUI_HandleEmuCommands(TestCommandState* testfn)
 {
 	bool taseditor = FCEUMOV_Mode(MOVIEMODE_TASEDITOR);
-	for(i=0; i<NUM_EMU_CMDS; ++i)
+	for(size_t i=0; i<NUM_EMU_CMDS; ++i)
 	{
 		int new_state;
 		int old_state = FCEUI_CommandTable[i].state;
@@ -1008,7 +1044,7 @@ static void CommandSelectSaveSlot(void)
 {
 	if (FCEUMOV_Mode(MOVIEMODE_TASEDITOR))
 	{
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 		handleEmuCmdByTaseditor(execcmd);
 #endif
 	} else
@@ -1026,7 +1062,7 @@ static void CommandStateSave(void)
 {
 	if (FCEUMOV_Mode(MOVIEMODE_TASEDITOR))
 	{
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 		handleEmuCmdByTaseditor(execcmd);
 #endif
 	} else
@@ -1047,7 +1083,7 @@ static void CommandStateLoad(void)
 {
 	if (FCEUMOV_Mode(MOVIEMODE_TASEDITOR))
 	{
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 		handleEmuCmdByTaseditor(execcmd);
 #endif
 	} else
@@ -1110,7 +1146,7 @@ void LagCounterToggle(void)
 
 static void LaunchTasEditor(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	extern bool enterTASEditor();
 	enterTASEditor();
 #endif
@@ -1118,56 +1154,56 @@ static void LaunchTasEditor(void)
 
 static void LaunchMemoryWatch(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	CreateMemWatch();
 #endif
 }
 
 static void LaunchDebugger(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	DoDebug(0);
 #endif
 }
 
 static void LaunchNTView(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	DoNTView();
 #endif
 }
 
 static void LaunchPPU(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	DoPPUView();
 #endif
 }
 
 static void LaunchHex(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	DoMemView();
 #endif
 }
 
 static void LaunchTraceLogger(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	DoTracer();
 #endif
 }
 
 static void LaunchCodeDataLogger(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	DoCDLogger();
 #endif
 }
 
 static void LaunchCheats(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	extern HWND hCheat;
 	ConfigCheats(hCheat);
 #endif
@@ -1175,7 +1211,7 @@ static void LaunchCheats(void)
 
 static void LaunchRamWatch(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	extern void OpenRamWatch();	//adelikat: Blah blah hacky, I know
 	OpenRamWatch();
 #endif
@@ -1183,14 +1219,14 @@ static void LaunchRamWatch(void)
 
 static void LaunchRamSearch(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	extern void OpenRamSearch();
 	OpenRamSearch();
 #endif
 }
 
 static void RamSearchOpLT(void) {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	if (GameInfo)
 	{
 		extern void SetSearchType(int SearchType);
@@ -1202,7 +1238,7 @@ static void RamSearchOpLT(void) {
 }
 
 static void RamSearchOpGT(void) {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	if (GameInfo)
 	{
 		extern void SetSearchType(int SearchType);
@@ -1214,7 +1250,7 @@ static void RamSearchOpGT(void) {
 }
 
 static void RamSearchOpLTE(void) {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	if (GameInfo)
 	{
 		extern void SetSearchType(int SearchType);
@@ -1226,7 +1262,7 @@ static void RamSearchOpLTE(void) {
 }
 
 static void RamSearchOpGTE(void) {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	if (GameInfo)
 	{
 		extern void SetSearchType(int SearchType);
@@ -1238,7 +1274,7 @@ static void RamSearchOpGTE(void) {
 }
 
 static void RamSearchOpEQ(void) {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	if (GameInfo)
 	{
 		extern void SetSearchType(int SearchType);
@@ -1250,7 +1286,7 @@ static void RamSearchOpEQ(void) {
 }
 
 static void RamSearchOpNE(void) {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	if (GameInfo)
 	{
 		extern void SetSearchType(int SearchType);
@@ -1261,9 +1297,21 @@ static void RamSearchOpNE(void) {
 #endif
 }
 
+extern int globalCheatDisabled;
+extern unsigned int FrozenAddressCount;
+static void ToggleCheats()
+{
+	FCEUI_GlobalToggleCheat(globalCheatDisabled);
+	FCEU_DispMessage("%d cheats active", 0, FrozenAddressCount);
+	#ifdef __WIN_DRIVER__
+	UpdateCheatRelatedWindow();
+	UpdateCheatListGroupBoxUI();
+	#endif
+}
+
 static void DebuggerStepInto()
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	if (GameInfo)
 	{
 		extern void DoDebuggerStepInto();
@@ -1279,7 +1327,7 @@ static void FA_SkipLag(void)
 
 static void OpenRom(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	extern HWND hAppWnd;
 	LoadNewGamey(hAppWnd, 0);
 #endif
@@ -1287,14 +1335,14 @@ static void OpenRom(void)
 
 static void CloseRom(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	CloseGame();
 #endif
 }
 
 void ReloadRom(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	if (FCEUMOV_Mode(MOVIEMODE_TASEDITOR))
 	{
 		// load most recent project
@@ -1317,23 +1365,20 @@ static void MovieSubtitleToggle(void)
 
 static void UndoRedoSavestate(void)
 {
-	// FIXME this will always evaluate to true, should this be
-	// if (*lastSavestateMade...) to check if it holds a string or just
-	// a '\0'?
-	if (lastSavestateMade && (undoSS || redoSS))
+	if ( !lastSavestateMade.empty() && (undoSS || redoSS))
 		SwapSaveState();
 }
 
 static void FCEUI_DoExit(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	DoFCEUExit();
 #endif
 }
 
 void ToggleFullscreen()
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	extern int SetVideoMode(int fs);		//adelikat: Yeah, I know, hacky
 	extern void UpdateCheckedMenuItems();
 
@@ -1349,21 +1394,34 @@ void ToggleFullscreen()
 
 static void TaseditorRewindOn(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	mustRewindNow = true;
 #endif
 }
 static void TaseditorRewindOff(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	mustRewindNow = false;
 #endif
 }
 
 static void TaseditorCommand(void)
 {
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	if (FCEUMOV_Mode(MOVIEMODE_TASEDITOR))
 		handleEmuCmdByTaseditor(execcmd);
 #endif
 }
+
+/**
+* Function to get command info entry by command number
+**/
+EMUCMDTABLE* GetEmuCommandById(int cmd)
+{
+	for (size_t i = 0; i<NUM_EMU_CMDS; ++i)
+	{
+		if (FCEUI_CommandTable[i].cmd == cmd)
+			return &FCEUI_CommandTable[i];
+	}
+	return NULL;
+}
