@@ -21,6 +21,7 @@
 #include "utils/memory.h"
 #include "utils/xstring.h"
 #include <sstream>
+#include <algorithm>
 
 #ifdef CREATE_AVI
 #include "drivers/videolog/nesvideos-piece.h"
@@ -28,6 +29,8 @@
 
 #ifdef WIN32
 #include <windows.h>
+
+#ifdef __WIN_DRIVER__
 #include "./drivers/win/common.h"
 #include "./drivers/win/window.h"
 extern void AddRecentMovieFile(const char *filename);
@@ -35,6 +38,12 @@ extern void AddRecentMovieFile(const char *filename);
 extern bool mustEngageTaseditor;
 #endif
 
+#endif
+
+#ifdef __QT_DRIVER__
+#include "./drivers/Qt/TasEditor/TasEditorWindow.h"
+#endif
+
 extern int RAMInitOption;
 extern int RAMInitSeed;
 
@@ -105,7 +114,7 @@ SFORMAT FCEUMOV_STATEINFO[]={
 	{ 0 }
 };
 
-char curMovieFilename[512] = {0};
+std::string curMovieFilename;
 MovieData currMovieData;
 MovieData defaultMovieData;
 int currRerecordCount; // Keep the global value
@@ -397,12 +406,13 @@ MovieData::MovieData()
 	, fds(false)
 	, palFlag(false)
 	, PPUflag(false)
-	, RAMInitOption(0)
-	, RAMInitSeed(0)
 	, rerecordCount(0)
 	, binaryFlag(false)
 	, loadFrameCount(-1)
+	, fourscore(false)
 	, microphone(false)
+	, RAMInitOption(0)
+	, RAMInitSeed(0)
 {
 	memset(&romChecksum,0,sizeof(MD5DATA));
 }
@@ -619,7 +629,7 @@ static void LoadFM2_binarychunk(MovieData& movieData, EMUFILE* fp, int size)
 	fp->fseek(curr,SEEK_SET);
 
 	//the amount todo is the min of the limiting size we received and the remaining contents of the file
-	int todo = std::min(size, flen);
+	int todo = std::min<int>(size, flen);
 
 	int numRecords = todo/recordsize;
 	if (movieData.loadFrameCount!=-1 && movieData.loadFrameCount<numRecords)
@@ -683,7 +693,7 @@ bool LoadFM2(MovieData& movieData, EMUFILE* fp, int size, bool stopAfterHeader)
 		{
 			LoadFM2_binarychunk(movieData, fp, size);
 			return true;
-		} else if (isnewline && movieData.loadFrameCount == movieData.records.size())
+		} else if (isnewline && static_cast<size_t>(movieData.loadFrameCount) == movieData.records.size())
 			// exit prematurely if loaded the specified amound of records
 			return true;
 		switch(state)
@@ -730,6 +740,9 @@ bool LoadFM2(MovieData& movieData, EMUFILE* fp, int size, bool stopAfterHeader)
 			state = VALUE;
 			if(isnewline) goto commit;
 			value += c;
+			break;
+		default:
+			break;
 		}
 		goto done;
 
@@ -794,7 +807,10 @@ static EMUFILE *openRecordingMovie(const char* fname)
 		FCEU_PrintError("Error opening movie output file: %s", fname);
 		return NULL;
 	}
-	strcpy(curMovieFilename, fname);
+	if ( fname != curMovieFilename.c_str() )
+	{
+		curMovieFilename.assign(fname);
+	}
 
 	return osRecordingMovie;
 }
@@ -814,7 +830,7 @@ static void RedumpWholeMovieFile(bool justToggledRecording = false)
 	bool recording = (movieMode == MOVIEMODE_RECORD);
 	assert((NULL != osRecordingMovie) == (recording != justToggledRecording) && "osRecordingMovie should be consistent with movie mode!");
 
-	if (NULL == openRecordingMovie(curMovieFilename))
+	if (NULL == openRecordingMovie(curMovieFilename.c_str()))
 		return;
 
 	currMovieData.dump(osRecordingMovie, false/*currMovieData.binaryFlag*/, recording);
@@ -862,11 +878,11 @@ static void OnMovieClosed()
 {
 	assert(movieMode == MOVIEMODE_INACTIVE);
 
-	curMovieFilename[0] = 0;			//No longer a current movie filename
+	curMovieFilename.clear();			//No longer a current movie filename
 	freshMovie = false;					//No longer a fresh movie loaded
 	if (bindSavestate) AutoSS = false;	//If bind movies to savestates is true, then there is no longer a valid auto-save to load
 
-#ifdef WIN32
+#if defined(__WIN_DRIVER__)
 	SetMainWindowText();
 #endif
 }
@@ -920,7 +936,10 @@ void FCEUMOV_CreateCleanMovie()
 	currMovieData = MovieData();
 	currMovieData.palFlag = FCEUI_GetCurrentVidSystem(0,0)!=0;
 	currMovieData.romFilename = FileBase;
-	currMovieData.romChecksum = GameInfo->MD5;
+	if ( GameInfo )
+	{
+		currMovieData.romChecksum = GameInfo->MD5;
+	}
 	currMovieData.guid.newGuid();
 	currMovieData.fourscore = FCEUI_GetInputFourscore();
 	currMovieData.microphone = FCEUI_GetInputMicrophone();
@@ -975,7 +994,7 @@ bool MovieData::loadSaveramFrom(std::vector<uint8>* buf)
 			return false;
 		}
 
-		if(currCartInfo->SaveGameLen[i] != len)
+		if(currCartInfo->SaveGameLen[i] != static_cast<unsigned int>(len))
 		{
 			FCEU_PrintError("movie battery load mismatch 3");
 			return false;
@@ -1024,7 +1043,7 @@ bool FCEUI_LoadMovie(const char *fname, bool _read_only, int _pauseframe)
 
 	currMovieData = MovieData();
 
-	strcpy(curMovieFilename, fname);
+	curMovieFilename.assign(fname);
 	FCEUFILE *fp = FCEU_fopen(fname,0,"rb",0);
 	if (!fp) return false;
 	if(fp->isArchive() && !_read_only) {
@@ -1032,7 +1051,7 @@ bool FCEUI_LoadMovie(const char *fname, bool _read_only, int _pauseframe)
 		return true;	//adelikat: file did not fail to load, so return true (false is only for file not exist/unable to open errors
 	}
 
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	//Fix relative path if necessary and then add to the recent movie menu
 	extern std::string BaseDirectory;
 
@@ -1099,7 +1118,7 @@ bool FCEUI_LoadMovie(const char *fname, bool _read_only, int _pauseframe)
 	else
 		FCEU_DispMessage("Replay started Read+Write.",0);
 
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	SetMainWindowText();
 #endif
 
@@ -1129,7 +1148,7 @@ void FCEUI_SaveMovie(const char *fname, EMOVIE_FLAG flags, std::wstring author)
 	if (NULL == openRecordingMovie(fname))
 		return;
 
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	//Add to the recent movie menu
 	AddRecentMovieFile(fname);
 #endif
@@ -1176,7 +1195,7 @@ void FCEUI_SaveMovie(const char *fname, EMOVIE_FLAG flags, std::wstring author)
 //either dumps the current joystick state or loads one state from the movie
 void FCEUMOV_AddInputState()
 {
-#ifdef _WIN32
+#if defined(__WIN_DRIVER__) || defined(__QT_DRIVER__)
 	if (movieMode == MOVIEMODE_TASEDITOR)
 	{
 		// if movie length is less or equal to currFrame, pad it with empty frames
@@ -1205,7 +1224,11 @@ void FCEUMOV_AddInputState()
 		if (mr->command_fds_select())
 			FCEU_FDSSelect();
 		if (mr->command_vs_insertcoin())
-			FCEU_VSUniCoin();
+			FCEU_VSUniCoin(0);
+		if (mr->command_vs_insertcoin2())
+			FCEU_VSUniCoin(1);
+		if (mr->command_vs_service())
+			FCEU_VSUniService();
 		_currCommand = 0;
 	} else
 #endif
@@ -1233,14 +1256,18 @@ void FCEUMOV_AddInputState()
 			if(mr->command_fds_select())
 				FCEU_FDSSelect();
 			if (mr->command_vs_insertcoin())
-				FCEU_VSUniCoin();
+				FCEU_VSUniCoin(0);
+			if (mr->command_vs_insertcoin2())
+				FCEU_VSUniCoin(1);
+			if (mr->command_vs_service())
+				FCEU_VSUniService();
 
 			joyports[0].load(mr);
 			joyports[1].load(mr);
 		}
 
 		//if we are on the last frame, then pause the emulator if the player requested it
-		if (currFrameCounter == currMovieData.records.size()-1)
+		if ( static_cast<size_t>(currFrameCounter) == currMovieData.records.size()-1)
 		{
 			if(FCEUD_PauseAfterPlayback())
 			{
@@ -1311,6 +1338,8 @@ void FCEUMOV_AddCommand(int cmd)
 		case FCEUNPCMD_FDSINSERT: cmd = MOVIECMD_FDS_INSERT; break;
 		case FCEUNPCMD_FDSSELECT: cmd = MOVIECMD_FDS_SELECT; break;
 		case FCEUNPCMD_VSUNICOIN: cmd = MOVIECMD_VS_INSERTCOIN; break;
+		case FCEUNPCMD_VSUNICOIN2: cmd = MOVIECMD_VS_INSERTCOIN2; break;
+		case FCEUNPCMD_VSUNISERVICE: cmd = MOVIECMD_VS_SERVICE; break;
 		// all other netplay commands (e.g. FCEUNPCMD_VSUNIDIP0) are not supported by movie recorder for now
 		default: return;
 	}
@@ -1410,7 +1439,7 @@ bool FCEUMOV_ReadState(EMUFILE* is, uint32 size)
 	{
 		if (currMovieData.loadFrameCount >= 0)
 		{
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 			int result = MessageBox(hAppWnd, "This movie is a TAS Editor project file.\nIt can be modified in TAS Editor only.\n\nOpen it in TAS Editor now?", "Movie Replay", MB_YESNO);
 			if (result == IDYES)
 				mustEngageTaseditor = true;
@@ -1419,7 +1448,7 @@ bool FCEUMOV_ReadState(EMUFILE* is, uint32 size)
 #endif
 			movie_readonly = true;
 		}
-		if (FCEU_isFileInArchive(curMovieFilename))
+		if (FCEU_isFileInArchive(curMovieFilename.c_str()))
 		{
 			//a little rule: cant load states in read+write mode with a movie from an archive.
 			//so we are going to switch it to readonly mode in that case
@@ -1498,7 +1527,7 @@ bool FCEUMOV_ReadState(EMUFILE* is, uint32 size)
 		if(tempMovieData.guid != currMovieData.guid)
 		{
 			//mbg 8/18/08 - this code  can be used to turn the error message into an OK/CANCEL
-			#ifdef WIN32
+			#ifdef __WIN_DRIVER__
 				std::string msg = "There is a mismatch between savestate's movie and current movie.\ncurrent: " + currMovieData.guid.toString() + "\nsavestate: " + tempMovieData.guid.toString() + "\n\nThis means that you have loaded a savestate belonging to a different movie than the one you are playing now.\n\nContinue loading this savestate anyway?";
 				int result = MessageBox(hAppWnd, msg.c_str(), "Error loading savestate", MB_OKCANCEL);
 				if(result == IDCANCEL)
@@ -1554,10 +1583,10 @@ bool FCEUMOV_ReadState(EMUFILE* is, uint32 size)
 					//TODO: turn frame counter to red to get attention
 					if (!backupSavestates)	//If backups are disabled we can just resume normally since we can't restore so stop movie and inform user
 					{
-						FCEU_PrintError("Error: Savestate taken from a frame (%d) after the final frame in the savestated movie (%d) cannot be verified against current movie (%d). This is not permitted.\nUnable to restore backup, movie playback stopped.", currFrameCounter, tempMovieData.records.size() - 1, currMovieData.records.size() - 1);
+						FCEU_PrintError("Error: Savestate taken from a frame (%d) after the final frame in the savestated movie (%zi) cannot be verified against current movie (%zi). This is not permitted.\nUnable to restore backup, movie playback stopped.", currFrameCounter, tempMovieData.records.size() - 1, currMovieData.records.size() - 1);
 						FCEUI_StopMovie();
 					} else
-						FCEU_PrintError("Savestate taken from a frame (%d) after the final frame in the savestated movie (%d) cannot be verified against current movie (%d). This is not permitted.", currFrameCounter, tempMovieData.records.size() - 1, currMovieData.records.size() - 1);
+						FCEU_PrintError("Savestate taken from a frame (%d) after the final frame in the savestated movie (%zi) cannot be verified against current movie (%zi). This is not permitted.", currFrameCounter, tempMovieData.records.size() - 1, currMovieData.records.size() - 1);
 					return false;
 				}
 			}
@@ -1713,7 +1742,7 @@ void FCEUI_MovieToggleReadOnly()
 		strcpy(message, "Movie is now Read+Write");
 	
 	strcat(message, GetMovieModeStr());
-	FCEU_DispMessage(message,0);
+	FCEU_DispMessage("%s",0,message);
 }
 
 void FCEUI_MovieToggleRecording()
@@ -1757,7 +1786,7 @@ void FCEUI_MovieToggleRecording()
 
 	strcat(message, GetMovieModeStr());
 
-	FCEU_DispMessage(message, 0);
+	FCEU_DispMessage("%s",0,message);
 }
 
 void FCEUI_MovieInsertFrame()
@@ -1784,7 +1813,7 @@ void FCEUI_MovieInsertFrame()
 		strcat(message, GetMovieModeStr());
 	}
 
-	FCEU_DispMessage(message, 0);
+	FCEU_DispMessage("%s",0,message);
 }
 
 void FCEUI_MovieDeleteFrame()
@@ -1822,7 +1851,7 @@ void FCEUI_MovieDeleteFrame()
 		strcat(message, GetMovieModeStr());
 	}
 
-	FCEU_DispMessage(message, 0);
+	FCEU_DispMessage("%s",0,message);
 }
 
 void FCEUI_MovieTruncate()
@@ -1860,7 +1889,7 @@ void FCEUI_MovieTruncate()
 		strcat(message, GetMovieModeStr());
 	}
 
-	FCEU_DispMessage(message, 0);
+	FCEU_DispMessage("%s",0,message);
 }
 
 void FCEUI_MovieNextRecordMode()
@@ -1892,7 +1921,7 @@ void FCEUI_MoviePlayFromBeginning(void)
 {
 	if (movieMode == MOVIEMODE_TASEDITOR)
 	{
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 		handleEmuCmdByTaseditor(EMUCMD_MOVIE_PLAY_FROM_BEGINNING);
 #endif
 	} else if (movieMode != MOVIEMODE_INACTIVE)
@@ -1925,12 +1954,12 @@ void FCEUI_MoviePlayFromBeginning(void)
 			//currMovieData.loadSavestateFrom(&currMovieData.savestate); //TODO: make something like this work instead so it doesn't have to reload
 		}
 	}
-#ifdef WIN32
+#ifdef __WIN_DRIVER__
 	SetMainWindowText();
 #endif
 }
 
-string FCEUI_GetMovieName(void)
+std::string FCEUI_GetMovieName(void)
 {
 	return curMovieFilename;
 }
@@ -2024,9 +2053,9 @@ void FCEUI_CreateMovieFile(std::string fn)
 void FCEUI_MakeBackupMovie(bool dispMessage)
 {
 	//This function generates backup movie files
-	string currentFn;					//Current movie fillename
-	string backupFn;					//Target backup filename
-	string tempFn;						//temp used in back filename creation
+	std::string currentFn;					//Current movie fillename
+	std::string backupFn;					//Target backup filename
+	std::string tempFn;						//temp used in back filename creation
 	stringstream stream;
 	int x;								//Temp variable for string manip
 	bool exist = false;					//Used to test if filename exists
