@@ -7,7 +7,10 @@ extern int FCEUnetplay;
 #define FCEUNPCMD_POWER   0x02
 
 #define FCEUNPCMD_VSUNICOIN     0x07
-#define FCEUNPCMD_VSUNIDIP0	0x08
+#define FCEUNPCMD_VSUNIDIP0		0x08
+#define FCEUNPCMD_VSUNICOIN2	0x20
+#define FCEUNPCMD_VSUNISERVICE	0x21
+
 #define FCEUNPCMD_FDSINSERTx	0x10
 #define FCEUNPCMD_FDSINSERT	0x18
 //#define FCEUNPCMD_FDSEJECT	0x19
