@@ -41,6 +41,7 @@
 #include <cstring>
 
 bool force_grayscale = false;
+pal *grayscaled_palo = NULL;
 
 pal palette_game[64*8]; //custom palette for an individual game. (formerly palettei)
 pal palette_user[64*8]; //user's overridden palette (formerly palettec)
@@ -86,6 +87,7 @@ pal *palo = NULL;
 	(type) (y + to_rgb [4] * i + to_rgb [5] * q)\
 )
 
+FCEU_MAYBE_UNUSED
 static void ApplyDeemphasisNTSC(int entry, u8& r, u8& g, u8& b)
 {
 				static float const to_float = 1.0f / 0xFF;
@@ -176,7 +178,7 @@ int bisqwit_wave(int p, int color) { return (color+p+8)%12 < 6; }
 static void ApplyDeemphasisBisqwit(int entry, u8& r, u8& g, u8& b)
 {
 	if(entry<64) return;
-	int myr, myg, myb;
+	int myr=0, myg=0, myb=0;
 	// The input value is a NES color index (with de-emphasis bits).
 	// We need RGB values. Convert the index into RGB.
 	// For most part, this process is described at:
@@ -251,6 +253,7 @@ static void ApplyDeemphasisBisqwit(int entry, u8& r, u8& g, u8& b)
 }
 
 //classic algorithm
+FCEU_MAYBE_UNUSED
 static void ApplyDeemphasisClassic(int entry, u8& r, u8& g, u8& b)
 {
 	//DEEMPH BITS MAY BE ORDERED WRONG. PLEASE CHECK
@@ -526,6 +529,29 @@ static void ChoosePalette(void)
 		//need to calcualte a deemph on the fly.. sorry. maybe support otherwise later
 		ApplyDeemphasisComplete(palo);
 	}
+	if (force_grayscale)
+	{
+		// need to apply grayscale filter
+		// allocate memory for grayscale palette
+		if (grayscaled_palo == NULL)
+			grayscaled_palo = (pal*)malloc(sizeof(pal) * 64 * 8);
+		// make every color grayscale
+		for (int x = 0; x < 64 * 8; x++)
+		{
+			uint8 gray = ((float)palo[x].r * 0.299 + (float)palo[x].g * 0.587 + (float)palo[x].b * 0.114);
+			grayscaled_palo[x].r = gray;
+			grayscaled_palo[x].g = gray;
+			grayscaled_palo[x].b = gray;
+		}
+		// apply new palette
+		palo = grayscaled_palo;
+	}
+	else if (grayscaled_palo != NULL)
+	{
+		// free allocated memory if the grayscale filter is not used anymore
+		free(grayscaled_palo);
+		grayscaled_palo = NULL;
+	}
 }
 
 void WritePalette(void)
